@@ -1,10 +1,28 @@
 /* exprtype.c -- propagates datatype thru expressions.
 
-    Copyright (C) 1992 by Robert K. Moniot.
-    This program is free software.  Permission is granted to
-    modify it and/or redistribute it, retaining this notice.
-    No guarantees accompany this software.
+Copyright (c) 1998 by Robert K. Moniot.
 
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of ftnchek shall not be used
+in advertising or otherwise to promote the sale, use or other dealings in
+this Software without prior written authorization from the author.
 
 */
 
@@ -153,7 +171,7 @@ init_typesizes(VOID)
 		/* Only executes once.  Thus cannot change wordsize
 		   after processing starts. */
 {
-  static int trapdoor=FALSE;
+  static int trapdoor=FALSE, ptr_trapdoor=FALSE;
   if(trapdoor) {
     if(given_wordsize != local_wordsize) {
       (void)fprintf(stderr,
@@ -171,9 +189,23 @@ init_typesizes(VOID)
 	type_size[Z] = (BYTE)(4*given_wordsize);
       }
     }
+  }
+
+				/* Cray pointer size is set separately */
+  if(ptr_trapdoor) {
+    if(given_ptrsize != local_ptrsize) {
+      (void)fprintf(stderr,
+	      "\nSorry-Cannot change pointer size after processing starts");
+    }
+    given_ptrsize = local_ptrsize;
+  }
+  else {
+    ptr_trapdoor = TRUE;
+    local_ptrsize = given_ptrsize;
   }
 }
 
+
 	/* this routine propagates type in binary expressions */
 
 void
@@ -192,15 +224,17 @@ binexpr_type(term1,operator,term2,result)
 	size1 = term1->size,
 	size2 = term2->size,
         result_size;
+    int I_logop_I=FALSE;		/* for f90_mixed_type warning */
+
     if( ! is_computational_type(type1) ) {
 		syntax_error(term1->line_num,term1->col_num,
-			"noncomputational primary in expression:");
+			"numeric or character quantity expected:");
 		report_type(term1);
 		result_type = E;
     }
     else if( ! is_computational_type(type2) ) {
 		syntax_error(term2->line_num,term2->col_num,
-			"noncomputational primary in expression:");
+			"numeric or character quantity expected:");
 		report_type(term2);
 		result_type = E;
     }
@@ -229,8 +263,10 @@ binexpr_type(term1,operator,term2,result)
 	    case tok_NEQV:
 		if(type1 == L && type2 == L)
 		    result_type = L;
-		else if(type1 == I && type2 == I)
+		else if(type1 == I && type2 == I) {
 		    result_type = W+I;
+		    I_logop_I = TRUE;
+		}
 		else
 		    result_type = E;
 		break;
@@ -276,19 +312,28 @@ binexpr_type(term1,operator,term2,result)
 		report_mismatch(term1,operator,term2);
 	    }
 	    else if(result_type >= W) {	/* W result */
-	      if(f77_mixed_expr) {
-		warning(operator->line_num,operator->col_num,
-			"nonstandard type combination in expression:");
+				/* F90 warning suppressed for numeric exprs */
+	      if(f77_mixed_expr ||
+		 (f90_mixed_expr && ((type1>=L || type2>=L) || I_logop_I)) ) {
+		nonstandard(operator->line_num,operator->col_num,f90_mixed_expr,0);
+		msg_tail(": incompatible type combination in expression:");
 		report_mismatch(term1,operator,term2);
 	      }
 	      result_type -= W;
 	    }
-				/* Obscure standard rule */
-	    else if( (f77_mixed_expr) &&
-		  op == tok_concat && !in_assignment_stmt &&
-		  (size1 == size_ADJUSTABLE || size2 == size_ADJUSTABLE) ) {
-		nonstandard(operator->line_num,operator->col_num);
-		msg_tail("adjustable size cannot be concatenated here");
+				/* Obscure standard rule 6.2.2. We have to look
+				   for IN_ASSIGN flag in any of 3 places, since
+				   it gets prematurely turned off in
+				   fun_or_substr_handle production if one of
+				   operands is a substring expression.  */
+	    else if( f77_mixed_expr &&
+		     op == tok_concat &&
+		     !is_true(IN_ASSIGN,
+			 (term1->TOK_flags|operator->TOK_flags|term2->TOK_flags)) &&
+		 ((size1==size_ADJUSTABLE && !is_true(CONST_EXPR,term1->TOK_flags))
+	       || (size2==size_ADJUSTABLE && !is_true(CONST_EXPR,term2->TOK_flags))) ) {
+		nonstandard(operator->line_num,operator->col_num,0,0);
+		msg_tail(": adjustable size cannot be concatenated here");
 	    }
 	}
     }
@@ -474,6 +519,13 @@ if(debug_latest) {
 		make_true(PARAMETER_EXPR,result->TOK_flags);
     }
 
+			/* Keep track of dimension bound expressions */
+    if( is_true(DIM_BOUND_EXPR,term1->TOK_flags)
+	&& is_true(DIM_BOUND_EXPR,term2->TOK_flags) ) {
+      make_true(DIM_BOUND_EXPR,result->TOK_flags);
+    }
+
+
     if( is_true(EVALUATED_EXPR,term1->TOK_flags)
 	 && is_true(EVALUATED_EXPR,term2->TOK_flags) ) {
 		make_true(EVALUATED_EXPR,result->TOK_flags);
@@ -607,7 +659,7 @@ unexpr_type(operator,term1,result)
 
     if( ! is_computational_type(type1) ) {
 		syntax_error(term1->line_num,term1->col_num,
-			"noncomputational primary in expression:");
+			"numeric quantity expected:");
 		report_type(term1);
 		result_type = E;
     }
@@ -639,7 +691,7 @@ unexpr_type(operator,term1,result)
 		break;
 	}
 
-	if( type1 != E )
+	if( type1 != E ) {
 	    if( result_type == E) {
 		syntax_error(operator->line_num,operator->col_num,
 			"expression incompatible with operator:");
@@ -648,15 +700,16 @@ unexpr_type(operator,term1,result)
 		report_type(term1);
 	    }
 	    else if(result_type >= W) {
-	      if(f77_mixed_expr) {
-		warning(operator->line_num,operator->col_num,
-			"nonstandard type used with operator:");
+	      if(f77_mixed_expr || f90_mixed_expr) {
+		nonstandard(operator->line_num,operator->col_num,f90_mixed_expr,0);
+		msg_tail(": incompatible type used with operator:");
 		msg_tail(operator->src_text);
 		msg_tail("used with");
 		report_type(term1);
 	      }
 	      result_type -= W;
 	    }
+	}
     }
 
     result->TOK_type = type_byte(class_VAR, result_type);
@@ -666,6 +719,7 @@ unexpr_type(operator,term1,result)
 		/* Keep track of constant expressions */
     copy_flag(CONST_EXPR,result->TOK_flags,term1->TOK_flags);
     copy_flag(PARAMETER_EXPR,result->TOK_flags,term1->TOK_flags);
+    copy_flag(DIM_BOUND_EXPR,result->TOK_flags,term1->TOK_flags);
 
 		/* Remember if integer division was used */
     if(result_type == type_INTEGER)
@@ -711,13 +765,13 @@ assignment_stmt_type(term1,equals,term2)
 
     if( ! is_computational_type(type1) ) {
 		syntax_error(term1->line_num,term1->col_num,
-			"noncomputational primary in expression:");
+			"numeric or character quantity expected:");
 		report_type(term1);
 		result_type = E;
     }
     else if( ! is_computational_type(type2) ) {
 		syntax_error(term2->line_num,term2->col_num,
-			"noncomputational primary in expression:");
+			"numeric or character quantity expected:");
 		report_type(term2);
 		result_type = E;
     }
@@ -735,9 +789,9 @@ assignment_stmt_type(term1,equals,term2)
 	    }
 	    else {
 	      if(result_type >= W) {		/* W result */
-		if(f77_mixed_expr) {
-		  warning(equals->line_num,equals->col_num,
-		     "nonstandard type combination:");
+		if(f77_mixed_expr || f90_mixed_expr) {
+		  nonstandard(equals->line_num,equals->col_num,f90_mixed_expr,0);
+		  msg_tail(": incompatible type combination:");
 		  report_type(term2);
 		  msg_tail("assigned to");
 		  report_type(term1);
@@ -1333,7 +1387,7 @@ eval_intrins(defn,args)
      Token *args;
 #endif /* HAVE_STDC */
 {
-    int index;
+    intrins_flags_t index;
     index = (defn->intrins_flags & I_EVALUATED);
 
 				/* Args must be evaluated, except for LEN */
