@@ -1,23 +1,50 @@
 /* ftnchek.h:
 
 	Common definitions for Fortran Program Checker
+*/
 
-    Copyright (C) 1994 by Robert K. Moniot.
-    This program is free software.  Permission is granted to
-    modify it and/or redistribute it, retaining this notice.
-    No guarantees accompany this software.
+/*
+
+Copyright (c) 1999 by Robert K. Moniot.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of ftnchek shall not be used
+in advertising or otherwise to promote the sale, use or other dealings in
+this Software without prior written authorization from the author.
+
+Acknowledgement: the above permission notice was copied, with minor
+changes, from the XFree86 Project.
 
 */
 
 #define COPYRIGHT_DATE \
-"Copyright (C) 1994, 1995, 1996 by Robert K. Moniot"
+"Copyright (C) 1994, 1997, 1999 by Robert K. Moniot"
 #define COPYRIGHT_NOTICE \
 "This program is free software.  Permission is granted to\n\
 modify it and/or redistribute it, retaining this notice.\n\
 No guarantees accompany this software."
-#define VERSION_NUMBER		"FTNCHEK Version 2.9 April 1996"
-#define PATCHLEVEL		"Patch Level 4"
-#define PROJECT_VERSION		"P2" /* Project file format version number */
+#define VERSION_NUMBER		"FTNCHEK Version 2.11 July 1999"
+#define PATCHLEVEL		"Patch Level 2"
+#define PROJECT_VERSION		"P3" /* Project file format version number */
+
+#include "config.h"		/* Get system-specific information */
 
 		/* Define macro PROTO for declaring function prototypes.  If
 		   compiler doesn't accept prototypes, use -DNO_PROTOTYPES.
@@ -82,14 +109,19 @@ PROTO(void exit,(int));
 #endif
 #endif
 
-			/* Define standard home directory for VMS.  This
-			   is one place it looks for rc file.  You can
-			   also define SPECIAL_HOMEDIR for other systems
-			   such as MSDOS or Mac. */
+			/* Define standard home directory for VMS and MSDOS.
+			   This directory is one place ftnchek looks for the
+			   rc or ini file. */
 #ifdef VMS
 #ifndef SPECIAL_HOMEDIR
 #define SPECIAL_HOMEDIR "SYS$DISK:[]"
 #endif
+#endif
+
+#ifdef MSDOS
+#ifndef SPECIAL_HOMEDIR
+#define SPECIAL_HOMEDIR "c:\\lib"
+#endif
 #endif
 
 	/* Define the Unix-style startup file name and a non-Unix-style
@@ -172,30 +204,15 @@ PROTO(void exit,(int));
 
 
 
-				/* The following macros define the default
-				   value of the -source setting */
 
-#ifndef DEC_TABS	/* DEC-style ugly tabbed source is off by default */
-#define DEC_TABS 0
+			/* This is the default limit on "cascades" of error
+			   messages where the trailer "etc..." is given
+			   instead of printing them all.  The limit can be
+			   changed at run time by -error=num.
+			 */
+#ifndef DEF_ERROR_CASCADE_LIMIT
+#define DEF_ERROR_CASCADE_LIMIT 3
 #endif
-			/* VMS-style INCLUDE supports defaulting
-			   extension, /NOLIST feature.  It is default
-			   for VMS, not for other systems. */
-#ifndef VMS_INCLUDE
-#ifdef VMS
-#define VMS_INCLUDE 1
-#else
-#define VMS_INCLUDE 0
-#endif
-#endif
-
-#ifndef UNIX_BACKSLASH	/* UNIX backslash escape sequences in strings */
-#define UNIX_BACKSLASH 0
-#endif
-
-			/* This is the default -source value */
-#define DEF_SOURCE_FORMAT (DEC_TABS+2*VMS_INCLUDE+4*UNIX_BACKSLASH)
-
 
 	/* VCG support is now standard. If you don't want it, you can
 	   eliminate it by defining NO_VCG_SUPPORT.  You can customize
@@ -211,9 +228,44 @@ PROTO(void exit,(int));
 #endif
 
 	/* The following defines control the default status of
-	   warnings controlled by -f77, -port, -pretty, and -trunc.
+	   warnings controlled by -arg, -f77/90/95, -port, -pretty, and -trunc.
 	 */
 
+
+			/* -arguments options default to ARGCHECK_ALL.  Define
+			   LAX_ARGCHECK to make -arguments=none the default.
+			   Otherwise -arguments=all is default.
+			*/
+#ifdef LAX_ARGCHECK
+#define ARGCHECK_ALL FALSE
+#else
+#define ARGCHECK_ALL TRUE
+#endif
+
+
+			/* -array options default to ARRAYCHECK_ALL.  Define
+			   LAX_ARRAYCHECK to make -array=none the default.
+			   Otherwise -array=all is default.
+			*/
+#ifdef LAX_ARRAYCHECK
+#define ARRAYCHECK_ALL FALSE
+#else
+#define ARRAYCHECK_ALL TRUE
+#endif
+
+
+			/* -common options default to COMCHECK_ALL, except for
+			   volatile, which defaults to FALSE.  Define
+			   LAX_COMCHECK to make -common=none the default.
+			   Otherwise -common=type,length,exact is default.
+			*/
+#ifdef LAX_COMCHECK
+#define COMCHECK_ALL FALSE
+#else
+#define COMCHECK_ALL TRUE
+#endif
+
+
 			/* -f77 options default to F77_ALL.  Define
 			   STRICT_SYNTAX to make -f77=all the default.
 			   Otherwise -f77=none is default.
@@ -222,6 +274,24 @@ PROTO(void exit,(int));
 #define F77_ALL 1
 #else
 #define F77_ALL 0
+#endif
+
+			/* -f90 options default to none unless
+			   F90_SYNTAX is defined.
+			*/
+#ifdef F90_SYNTAX
+#define F90_ALL 1
+#else
+#define F90_ALL 0
+#endif
+
+			/* -f95 options default to none unless
+			   F95_SYNTAX is defined.
+			*/
+#ifdef F95_SYNTAX
+#define F95_ALL 1
+#else
+#define F95_ALL 0
 #endif
 
 			/* -portability options default to PORT_ALL.  Define
@@ -244,6 +314,39 @@ PROTO(void exit,(int));
 #define PRETTY_ALL TRUE
 #endif
 
+			/* -source options are defined here.  Since
+			   -source=all is unlikely to be desired, each
+			   option has its own macro for default value */
+
+#ifndef DEC_TABS	/* DEC-style ugly tabbed source is off by default */
+#define DEC_TABS FALSE
+#endif
+			/* VMS-style INCLUDE supports defaulting
+			   extension, /NOLIST feature.  It is default
+			   for VMS, not for other systems. */
+#ifndef VMS_INCLUDE
+#ifdef VMS
+#define VMS_INCLUDE TRUE
+#else
+#define VMS_INCLUDE FALSE
+#endif
+#endif
+
+#ifndef UNIX_BACKSLASH	/* UNIX backslash escape sequences in strings */
+#define UNIX_BACKSLASH FALSE
+#endif
+
+		/* standard parameters take data type from value */
+#ifndef PARAMETER_IMPLICIT_TYPE
+#define PARAMETER_IMPLICIT_TYPE FALSE
+#endif
+
+		/* DEC-style parameters get data type in standard way */
+#ifndef DEC_PARAMETER_STANDARD_TYPE
+#define DEC_PARAMETER_STANDARD_TYPE FALSE
+#endif
+
+
 			/* -truncation options default to TRUNC_ALL.  Define
 			   LAX_TRUNCATION to make -trunc=none the default.
 			   Otherwise -trunc=all is default.
@@ -254,76 +357,95 @@ PROTO(void exit,(int));
 #define TRUNC_ALL TRUE
 #endif
 
+			/* -usage options default to USAGE_ALL.  Define
+			   LAX_USAGE to make -usage=none the default.
+			   Otherwise -usage=all is default.
+			*/
+#ifdef LAX_USAGE
+#define USAGE_ALL FALSE
+#else
+#define USAGE_ALL TRUE
+#endif
 
-		/* If STANDARD_INTRINSICS is defined, only F77
-		   standard intrinsic functions (plus double complex
-		   flavors of standard ones) will be known to ftnchek.
-		   Otherwise there are three classes of nonstandard
-		   intrinsics: common, Unix flavor, and VMS flavor.
-		   Common ones will be recognized if -intrinsic=
-		   nonzero.  -intrinsic= 2 or 3 specify UNIX or VMS
-		   rsptly, unless suppressed with NO_UNIX_INTRINSICS
-		   or NO_VMS_INTRINSICS.  */
+		/* The macros in the following section control whether
+		   certain groups of nonstandard intrinsics are compiled
+		   into ftnchek.  By turning off all these macros using the
+		   STANDARD_INTRINSICS define or selectively by using the
+		   NO_ prefix, some efficiency is gained but these intrinsics
+		   will then never be recognized by ftnchek, regardless of
+		   the -intrinsic setting.  Turn these off only if you are
+		   a purist who will never use non-F77 intrinsics.
+		 */
 
 #ifndef STANDARD_INTRINSICS
 
-#define NONSTD_INTRINSICS	/* Common nonstandard intrinsic functions */
+#ifndef NO_EXTRA_INTRINSICS
+#define EXTRA_INTRINSICS	/* Common nonstandard intrinsic functions */
+#endif
 
 #ifndef NO_UNIX_INTRINSICS
 #define UNIX_INTRINSICS		/* UNIX intrinsic functions */
 #endif
-	/* Note: RAND syntax varies.  Define RAND_NO_ARG to make ftnchek
-	   expect X=RAND(). By default ftnchek expects X=RAND(ISEED).
-	 */
 
 #ifndef NO_VMS_INTRINSICS
 #define VMS_INTRINSICS		/* VMS intrinsic functions */
 #endif
 
-#endif /* not STANDARD_INTRINSICS */
 
-			/* Define the default -intrinsic setting */
 
-				/* Define which set to use */
-#ifdef UNIX	/* Unix default: accept Unix intrinsics */
-#define DEF_INTRINSIC_SET 2
-#endif
 
-#ifdef VMS	/* VMS default: accept VMS intrinsics */
+		/* The macros in the next section define the default
+		   value of the -intrinsic settings governing which
+		   intrinsic groups to recognize and how to treat RAND
+		   and IARGC, which may take no argument or one dummy
+		   argument on different platforms.
+		 */
+
+			/* Define which sets to recognize by default:
+			     bit 0 = commonly available nonstandard intrinsics
+			     bit 1 = some Unix intrinsics
+			     bit 2 = some VMS intrinsics
+			 */
+
+#ifdef UNIX	/* Unix default: accept Common & Unix intrinsics */
 #define DEF_INTRINSIC_SET 3
 #endif
 
-#ifndef DEF_INTRINSIC_SET	/* Neither one: use only common set */
+#ifdef VMS	/* VMS default: accept Common & VMS intrinsics */
+#define DEF_INTRINSIC_SET 5
+#endif
+
+#ifndef DEF_INTRINSIC_SET	/* Otherwise default = only common set */
 #define DEF_INTRINSIC_SET 1
 #endif
 
-				/* Define which RAND form to use */
+				/* Define which RAND form to use.
+				     bit 0 = allow no-arg form
+				     bit 1 = allow one-arg form
+				 */
+
 #ifdef RAND_NO_ARG	/*RAND() form*/
-#define DEF_INTRINSIC_RAND 0
+#define DEF_INTRINSIC_RAND 1
 #else
 #ifdef RAND_ONE_ARG	/*RAND(ISEED) form*/
-#define DEF_INTRINSIC_RAND 1
-#else				/* Allow either form */
 #define DEF_INTRINSIC_RAND 2
+#else				/* Default: allow either form */
+#define DEF_INTRINSIC_RAND 3
 #endif
 #endif
 
 				/* Define which IARGC form to use */
 #ifdef IARGC_NO_ARG
-#define DEF_INTRINSIC_IARG 0
+#define DEF_INTRINSIC_IARGC 1
 #else
 #ifdef IARGC_ONE_ARG
-#define DEF_INTRINSIC_IARG 1
-#else  /* default is to allow 0 or 1 */
-#define DEF_INTRINSIC_IARG 2
+#define DEF_INTRINSIC_IARGC 2
+#else				/* Default: allow either form */
+#define DEF_INTRINSIC_IARGC 3
 #endif
 #endif
-				/* Now define the default setting from
-				   the above choices. */
-#define DEF_INTRINSIC_OPT (DEF_INTRINSIC_IARG*100 + DEF_INTRINSIC_RAND*10 \
-			   + DEF_INTRINSIC_SET)
-
 
+#endif /* not STANDARD_INTRINSICS */
 
 /*  Default BpW = 4 bytes per word, which matches many machines.
     This macro serves as the default value for -wordsize.
@@ -340,10 +462,18 @@ PROTO(void exit,(int));
     default sizes never match: e.g. passing REAL*8 where DOUBLE
     PRECISION expected.  None of this applies to CHARACTER data:
     the default size (1) is well-defined, and the standard does
-    not specify the ratio of sizeof(CHARACTER) to sizeof(REAL). */
+    not specify the ratio of sizeof(CHARACTER) to sizeof(REAL).
+
+    For the Cray pointer extension, we also define PTRSIZE as the size of
+    a pointer.  The default is the same as the wordsize.  This can be
+    changed at run time by -pointersize=nnn.
+*/
 
 #ifndef BpW
 #define BpW 4	/* Bytes per Word: might want to use sizeof(float) instead */
+#endif
+#ifndef PTRSIZE
+#define PTRSIZE BpW
 #endif
 
 		/* Define to tolerate embedded blanks in numeric consts unless
@@ -374,6 +504,7 @@ PROTO(void exit,(int));
 #define ALLOW_UNIX_BACKSLASH 1
 #define ALLOW_UNIX_CPP 1
 #define ALLOW_VMS_IO 1
+#define ALLOW_IBM_IO 1
 
 	/* If INLINE_COMMENT_CHAR is not defined, ! comments will cause
 	   ftnchek to choke.  F77_INLINE_COMMENT controls whether it
@@ -385,6 +516,10 @@ PROTO(void exit,(int));
 
 #define KEEP_ARG_NAMES 1	/* option was formerly experimental */
 
+#ifdef DEVELOPMENT		/* For maintaining the program */
+#define DEBUG_FORLEX
+#endif
+
 /*************************************************************************
 	 End section of defines to control ftnchek's behavior
 
@@ -467,7 +602,47 @@ PROTO(void exit,(int));
 
 #else  /* end if SMALL_MACHINE */
 
-#ifdef LARGE_MACHINE		/* use these if space is no problem */
+#ifdef MEDIUM_MACHINE		/* use these if default is too big */
+
+#ifndef HASHSZ
+#define HASHSZ 2002	/* MEDIUM_MACHINE Size of symbol hashtable */
+#endif
+#ifndef STRSPACESZ
+#define STRSPACESZ 10000 /* MEDIUM_MACHINE chunk size of string space */
+#endif
+#ifndef LOCSYMTABSZ
+#define LOCSYMTABSZ 800 /* MEDIUM_MACHINE Size of local symbol table */
+#endif
+#ifndef GLOBSYMTABSZ
+#define GLOBSYMTABSZ 1200 /* MEDIUM_MACHINE Size of global symbol table */
+#endif
+#ifndef PARAMINFOSPACESZ
+#define PARAMINFOSPACESZ 50 /* MEDIUM_MACHINE ParamInfo structs per chunk */
+#endif
+#ifndef TOKHEADSPACESZ
+#define TOKHEADSPACESZ 200 /* MEDIUM_MACHINE TokenListHeaders per chunk */
+#endif
+#ifndef TOKENSPACESZ
+#define TOKENSPACESZ 1000 /* MEDIUM_MACHINE tokens per token space chunk */
+#endif
+#ifndef ARGLISTHEADSZ
+#define ARGLISTHEADSZ	1500 /* MEDIUM_MACHINE argument headers per chunk */
+#endif
+#ifndef ARGLISTELTSZ
+#define ARGLISTELTSZ	5000	/* MEDIUM_MACHINE argument list elts per chunk */
+#endif
+#ifndef COMLISTHEADSZ
+#define COMLISTHEADSZ	1000 /* MEDIUM_MACHINE common list headers per chunk */
+#endif
+#ifndef COMLISTELTSZ
+#define COMLISTELTSZ	4000 /* MEDIUM_MACHINE common elts per chunk */
+#endif
+#ifndef PTRSPACESZ
+#define PTRSPACESZ	400 /* MEDIUM_MACHINE Max number of ptrs to arraydim */
+#endif
+
+#else		/* Default is now LARGE_MACHINE, OK for most modern machines */
+
 #ifndef HASHSZ		/* must be <= max int  */
 #define HASHSZ 20930	 /* LARGE_MACHINE Size of symbol hashtable */
 #endif
@@ -505,45 +680,6 @@ PROTO(void exit,(int));
 #define PTRSPACESZ	2000 /* LARGE_MACHINE Max number of ptrs to arraydim */
 #endif
 
-#else		/* Defaults: Use these for average-size applications */
-
-#ifndef HASHSZ
-#define HASHSZ 2002	/* Default Size of symbol hashtable */
-#endif
-#ifndef STRSPACESZ
-#define STRSPACESZ 10000 /* Default chunk size of string space */
-#endif
-#ifndef LOCSYMTABSZ
-#define LOCSYMTABSZ 800 /* Default Size of local symbol table */
-#endif
-#ifndef GLOBSYMTABSZ
-#define GLOBSYMTABSZ 1200 /* Default Size of global symbol table */
-#endif
-#ifndef PARAMINFOSPACESZ
-#define PARAMINFOSPACESZ 50 /* Default ParamInfo structs per chunk */
-#endif
-#ifndef TOKHEADSPACESZ
-#define TOKHEADSPACESZ 200 /* Default TokenListHeaders per chunk */
-#endif
-#ifndef TOKENSPACESZ
-#define TOKENSPACESZ 1000 /* Default tokens per token space chunk */
-#endif
-#ifndef ARGLISTHEADSZ
-#define ARGLISTHEADSZ	1500 /* Default argument headers per chunk */
-#endif
-#ifndef ARGLISTELTSZ
-#define ARGLISTELTSZ	5000	/* Default argument list elts per chunk */
-#endif
-#ifndef COMLISTHEADSZ
-#define COMLISTHEADSZ	1000 /* Default common list headers per chunk */
-#endif
-#ifndef COMLISTELTSZ
-#define COMLISTELTSZ	4000 /* Default common elts per chunk */
-#endif
-#ifndef PTRSPACESZ
-#define PTRSPACESZ	400 /* Default Max number of ptrs to arraydim */
-#endif
-
 #endif /* end if LARGE_MACHINE else */
 
 #endif/*end if SMALL_MACHINE else*/
@@ -616,22 +752,47 @@ SHARED char *main_filename;	/* name of file containing main program */
 #define OPT(Type,Name,Value) extern Type Name
 #endif
 
-				/* Define flag bits for call_tree_options */
-					/* First three are a 2-bit option */
-#define CALLTREE_PRINT		0x0001
-#define CALLTREE_REFLIST	0x0002
-#define CALLTREE_VCG		0x0003
+		/* These options are controlled by -arguments */
+OPT(int,argcheck_argnumber,ARGCHECK_ALL);/* Check for wrong no. of args */
+OPT(int,argcheck_arrayness,ARGCHECK_ALL);/* Check arg arrayness mismatches */
+OPT(int,argcheck_argtype,ARGCHECK_ALL);	 /* Check for arg type mismatches */
+OPT(int,argcheck_functype,ARGCHECK_ALL); /* Check for func type mismatches */
+		/* End of -arguments options */
 
-#define CALLTREE_NOPRUNE	0x0004 /* expand each node, not "see above" */
-#define CALLTREE_NOSORT		0x0008 /* program order, not alphabetized */
-#define CALLTREE_WITHARGS	0x0010 /* include args of calls */
+		/* These options are controlled by -array */
+OPT(int,arraycheck_dims,ARRAYCHECK_ALL);/* Check for mismatch no.of dims */
+OPT(int,arraycheck_size,ARRAYCHECK_ALL);/* Check for mismatch no.of elements */
+		/* End of -array options */
+
+		/* These options are controlled by -calltree */
+OPT(int,print_call_tree,FALSE);	/* Print the call tree */
+OPT(int,print_ref_list,FALSE);	/* Print reference (who-calls-who) list */
+#ifdef VCG_SUPPORT
+OPT(int,print_vcg_list,FALSE);	/* Print call graph in vcg format */
+#endif
+OPT(int,call_tree_prune,TRUE); /* do not repeat duplicates, use "see above"  */
+OPT(int,call_tree_sort,TRUE);  /* alphabetized, not program order */
+		/* End of -calltree options */
+
+		/* These options are controlled by -common */
+OPT(int,comcheck_type,COMCHECK_ALL);/* Check for type mismatch */
+OPT(int,comcheck_length,COMCHECK_ALL);/* Check for length mismatch  */
+OPT(int,comcheck_dims,COMCHECK_ALL);/* Check for array dim mismatch  */
+OPT(int,comcheck_by_name,COMCHECK_ALL);/* Check name-by-name */
+OPT(int,comcheck_volatile,FALSE); /* Assume blocks are "volatile" */
+		/* End of -common options */
 
 		/* These options are controlled by -f77 */
 OPT(int,f77_20_continue,F77_ALL);
 OPT(int,f77_accept_type,F77_ALL);
+OPT(int,f77_assignment,F77_ALL);
+OPT(int,f77_array_bounds,F77_ALL);
+OPT(int,f77_automatic_array,F77_ALL);
 OPT(int,f77_byte,F77_ALL);
 OPT(int,f77_cray_pointers,F77_ALL);
 OPT(int,f77_common_subprog_name,F77_ALL);
+OPT(int,f77_construct_name,F77_ALL);
+OPT(int,f77_cycle_exit,F77_ALL);
 OPT(int,f77_d_comment,F77_ALL);
 OPT(int,f77_dec_tabs,F77_ALL);
 OPT(int,f77_do_enddo,F77_ALL);
@@ -645,15 +806,18 @@ OPT(int,f77_include,F77_ALL);
 OPT(int,f77_inline_comment,F77_ALL);
 OPT(int,f77_internal_list_io,F77_ALL);
 OPT(int,f77_intrinsics,F77_ALL);
+OPT(int,f77_io_keywords,F77_ALL);
 OPT(int,f77_long_names,F77_ALL);
 OPT(int,f77_mixed_common,F77_ALL);
 OPT(int,f77_mixed_expr,F77_ALL);
 OPT(int,f77_namelist,F77_ALL);
 OPT(int,f77_overlength,F77_ALL);
+OPT(int,f77_param_implicit_type,F77_ALL);
 OPT(int,f77_param_intrinsic,F77_ALL);
 OPT(int,f77_param_noparen,F77_ALL);
 OPT(int,f77_quad_constants,F77_ALL);
 OPT(int,f77_quotemarks,F77_ALL);
+OPT(int,f77_relops,F77_ALL);
 OPT(int,f77_stmt_order,F77_ALL);
 OPT(int,f77_typeless_constants,F77_ALL);
 OPT(int,f77_typesize,F77_ALL);
@@ -661,9 +825,54 @@ OPT(int,f77_underscores,F77_ALL);
 OPT(int,f77_unix_backslash,F77_ALL);
 OPT(int,f77_unix_cpp,F77_ALL);
 OPT(int,f77_variable_format,F77_ALL);
-OPT(int,f77_vms_io,F77_ALL);
 		/* End of -f77 options */
 
+		/* These options are controlled by -f90 */
+OPT(int,f90_accept_type,F90_ALL);
+OPT(int,f90_byte,F90_ALL);
+OPT(int,f90_cray_pointers,F90_ALL);
+OPT(int,f90_d_comment,F90_ALL);
+OPT(int,f90_dec_tabs,F90_ALL);
+OPT(int,f90_dollarsigns,F90_ALL);
+OPT(int,f90_double_complex,F90_ALL);
+OPT(int,f90_format_dollarsigns,F90_ALL);
+OPT(int,f90_format_extensions,F90_ALL);
+OPT(int,f90_function_noparen,F90_ALL);
+OPT(int,f90_intrinsics,F90_ALL);
+OPT(int,f90_io_keywords,F90_ALL);
+OPT(int,f90_mixed_expr,F90_ALL);
+OPT(int,f90_overlength,F90_ALL);
+OPT(int,f90_param_implicit_type,F90_ALL);
+OPT(int,f90_param_noparen,F90_ALL);
+OPT(int,f90_quad_constants,F90_ALL);
+OPT(int,f90_stmt_order,F90_ALL);
+OPT(int,f90_typeless_constants,F90_ALL);
+OPT(int,f90_typesize,F90_ALL);
+OPT(int,f90_unix_backslash,F90_ALL);
+OPT(int,f90_unix_cpp,F90_ALL);
+OPT(int,f90_variable_format,F90_ALL);
+		/* End of -f90 options */
+
+		/* These options are controlled by -f95 */
+OPT(int,f95_real_do,F95_ALL);	/* real or d.p. DO variable */
+OPT(int,f95_pause,F95_ALL);	/* PAUSE stmt */
+OPT(int,f95_assign,F95_ALL);	/* ASSIGN stmt & kin */
+OPT(int,f95_Hedit,F95_ALL);	/* H edit descriptor */
+		/* End of -f95 options */
+
+		/* These options are controlled by -makedcls */
+OPT(int,dcl_declarations,FALSE);
+OPT(int,dcl_only_undeclared,FALSE);
+OPT(int,dcl_compact,FALSE);
+OPT(int,dcl_use_continuations,FALSE);
+OPT(int,dcl_keywords_lowercase,FALSE);
+OPT(int,dcl_vars_and_consts_lowercase,FALSE);
+OPT(int,dcl_excl_sftran3_internal_vars,FALSE);
+OPT(int,dcl_asterisk_comment_character,FALSE);
+OPT(int,dcl_lowercase_comment_character,FALSE);
+OPT(int,dcl_no_array_dimensions,FALSE);
+		/* End of -makedcls options */
+
 		/* These options are controlled by -portability */
 OPT(int,port_common_alignment,PORT_ALL);/* Common not in desc size order */
 OPT(int,port_backslash,PORT_ALL); 	/* Backslash used in standard way */
@@ -672,6 +881,7 @@ OPT(int,port_long_string,PORT_ALL);	/* Char string > 255 in length */
 OPT(int,port_hollerith,PORT_ALL);	/* Hollerith (except in FORMAT) */
 OPT(int,port_mixed_equiv,PORT_ALL);	/* Different types equivalenced */
 OPT(int,port_mixed_size,PORT_ALL);	/* sized, nonsized types mixed */
+OPT(int,port_param_implicit_type,PORT_ALL);/* Implicit param type != default */
 OPT(int,port_tabs,PORT_ALL);		/* Tabs in source */
 		/* End of -portability options */
 
@@ -686,6 +896,14 @@ OPT(int,pretty_no_space,PRETTY_ALL);    /* Space missing btw id and keyword */
 OPT(int,pretty_contin,PRETTY_ALL);	/* Continuation mark after comment */
 		/* End of -pretty options */
 
+		/* These options are controlled by -source */
+OPT(int,source_dec_tab,DEC_TABS); /*DEC Fortran tab-format*/
+OPT(int,source_vms_include,VMS_INCLUDE); /*VMS INCLUDE statement*/
+OPT(int,source_unix_backslash,UNIX_BACKSLASH); /*UNIX backslash escape*/
+OPT(int,source_dec_param_std_type,DEC_PARAMETER_STANDARD_TYPE);
+OPT(int,source_param_implicit,PARAMETER_IMPLICIT_TYPE);/* std params */
+		/* end of -source options */
+
 		/* These options are controlled by -truncation */
 OPT(int,trunc_int_div_real,TRUNC_ALL);	/* Int/int --> real */
 OPT(int,trunc_int_div_exponent,TRUNC_ALL);/* Int/int as exponentl */
@@ -699,16 +917,35 @@ OPT(int,trunc_promotion,TRUNC_ALL);	/* Low --> high precision */
 OPT(int,trunc_sigfigs,TRUNC_ALL);	/* Sngl const overspecified */
 				/* truncation warnings related to precision */
 #define trunc_precision (trunc_promotion||trunc_type_demotion||trunc_size_demotion)
-
 		/* End of -truncation options */
 
+		/* These options are controlled by -usage */
+OPT(int,usage_arg_modified,USAGE_ALL);		/* const or expr arg modified */
+OPT(int,usage_arg_alias_modified,USAGE_ALL);	/* arg same-as other modified */
+OPT(int,usage_array_alias_modified,USAGE_ALL);/* arg in same array as other modified */
+OPT(int,usage_arg_common_modified,USAGE_ALL);	/* arg in common modified */
+OPT(int,usage_array_common_modified,USAGE_ALL);	/* array arg in common modified */
+OPT(int,usage_com_block_unused,USAGE_ALL);	/* whole block declared but not used */
+OPT(int,usage_com_block_volatile,USAGE_ALL);	/* block may lose defn */
+OPT(int,usage_com_var_set_unused,USAGE_ALL);	/* set but not used */
+OPT(int,usage_com_var_uninitialized,USAGE_ALL);/* used but not set */
+OPT(int,usage_com_var_unused,USAGE_ALL);	/* declared but not used */
+OPT(int,usage_ext_declared_only,USAGE_ALL);	/* declared EXTERNAL but not defined or used */
+OPT(int,usage_ext_multiply_defined,USAGE_ALL);/* multiple definitions */
+OPT(int,usage_ext_undefined,USAGE_ALL);	/* used but not defined (= -external flag) */
+OPT(int,usage_ext_unused,USAGE_ALL);	/* defined but not used (like -library flag) */
+OPT(int,usage_var_set_unused,USAGE_ALL);	/* set but not used */
+OPT(int,usage_var_uninitialized,USAGE_ALL);	/* used before set */
+OPT(int,usage_var_unused,USAGE_ALL);		/* declared but not used */
+		/* End of -usage options */
+
+
 	/* The following flag is for warnings not covered by other flags,
 	   so that all warnings can be turned off by -nocheck. */
 OPT(int,misc_warn,TRUE);
 
 	/* The following variables are controlled by command line options */
 
-OPT(int,print_call_tree,FALSE);	/* Print the call tree */
 OPT(int,do_check,TRUE);		/* For -nocheck option */
 OPT(int,print_xref_list,FALSE);	/* Print subprogram cross-references */
 OPT(int,decls_required,FALSE);	/* List all undeclared identifiers */
@@ -725,16 +962,11 @@ OPT(int,do_list,FALSE);		/* Listing flag */
 OPT(int,novice_help,TRUE);	/* Extra help for novices */
 OPT(int,make_project_file,FALSE);/* Save symtab defns in .prj file */
 OPT(int,pure_functions,TRUE);	/* Assume functions are pure */
-OPT(int,print_ref_list,FALSE);	/* Print reference (who-calls-who) list */
 OPT(int,quiet,FALSE);		/* Less verbose output format */
 OPT(int,sixclash,FALSE);	/* To check if names unique in 1st 6 chars */
 OPT(int,print_topo_sort,FALSE);	/* Topological sort of modules */
 OPT(int,do_symtab,FALSE);	/* For symbol table printout */
-#ifdef VCG_SUPPORT
-OPT(int,print_vcg_list,FALSE);	/* Print call graph in vcg format */
-#endif
 OPT(int,print_version,FALSE);	/* Print version number and quit */
-OPT(int,volatile_flag,FALSE);	/* Assume volatile vars and comblocks */
 		/* Debugging flags */
 OPT(int,debug_latest,FALSE);	/* debug the latest addition */
 OPT(int,debug_glob_symtab,FALSE);/* global symtab contents */
@@ -742,65 +974,50 @@ OPT(int,debug_parser,FALSE);	/* grammar debug via DBG statements */
 OPT(int,debug_hashtab,FALSE);	/* hash table contents */
 OPT(int,debug_loc_symtab,FALSE); /* local symtab contents */
 OPT(int,show_resources,FALSE);	/* space avail and used */
+#ifdef DEBUG_FORLEX
 OPT(int,debug_lexer,FALSE);	/* list of tokens as scanned */
+#endif
 #ifdef MAIN
 extern int yydebug;		/* grammar debug via yydebug */
 #endif
 		/* Declare variables for commandline settings */
-OPT(int,argcheck_strictness,3);	/* Strictness for checking args */
-OPT(int,array_arg_check,3);/* Check array argument dims & size  */
 OPT(int,call_tree_options,0);	/* Sum of: 1 (print it), */
 				/* 2 (who-calls-who form), 4 (don't prune) */
 				/* 8 (program order), 16 (VCG format) */
-OPT(int,intrinsic_opt,DEF_INTRINSIC_OPT);/* 0=std, 1=common, 2=unix, 3=vms */
+
+
+#ifndef STANDARD_INTRINSICS
+				/* -intrinsic setting defaults */
+OPT(int,intrinsic_rand_no_argument,(DEF_INTRINSIC_RAND & 1)==1); /* rand form */
+OPT(int,intrinsic_rand_one_argument,(DEF_INTRINSIC_RAND & 2)==2);
+OPT(int,intrinsic_iargc_no_argument,(DEF_INTRINSIC_IARGC & 1)==1);/* iargc form */
+OPT(int,intrinsic_iargc_one_argument,(DEF_INTRINSIC_IARGC & 2)==2);
+OPT(int,intrinsic_set_extra,(DEF_INTRINSIC_SET & 1)==1);/* common intrinsics */
+OPT(int,intrinsic_set_unix,(DEF_INTRINSIC_SET & 2)==2);/* some unix intrinsics */
+OPT(int,intrinsic_set_vms,(DEF_INTRINSIC_SET & 4)==4);/* some vms intrinsics */
+#endif /* not STANDARD_INTRINSICS */
+
 OPT(int,max_stmt_col,72);	/* End of statement field ( <= MAXLINE )*/
-OPT(int,comcheck_strictness,3);	/* 0 (no check) to 3 (exact type & size) */
-OPT(int,make_dcls,0);		/* sum of: 0 (none), 1 (all), */
-				/* 2 (undeclared), 4 (compact), 8 (keywords */
-				/* lowercase), 16 (variables and constants */
-				/* lowercase), 32 (exclude sf3 internal */
-				/* variables), 64 (use * comment starter), */
-				/* 128 (use 'c' comment starter). */
-OPT(int,source_format,DEF_SOURCE_FORMAT);/* Sum of: 1=DEC tabs, */
-				/*   2=VMS style INCLUDE, 4=Unix backslash */
-OPT(int,usage_check,333);	/* Print set/used/notset checks */
+OPT(int,error_cascade_limit,DEF_ERROR_CASCADE_LIMIT);/* max errors in cascade */
+OPT(int,local_ptrsize,PTRSIZE);	/* Size of Cray pointer variables */
+OPT(int,given_ptrsize,PTRSIZE);	/* Size requested for Cray pointer variables */
+
 OPT(int,local_wordsize,BpW);	/* Bytes per word to use for default sizes */
 OPT(int,given_wordsize,BpW);	/* User's request as per -wordsize=n  */
 OPT(int,wrap_column,WRAP_COLUMN);/* For wrapping error messages */
 
-			/* Secondary control vars based on usage_check */
-OPT(int,var_usage_check,3);
-OPT(int,com_usage_check,3);
-OPT(int,ext_usage_check,3);
-
-		/* Secondary variable for selecting intrinsic function set */
-OPT(int,intrinsic_set,DEF_INTRINSIC_OPT%10);
 
 			/* Shorthands for checking control settings */
-#define check_args_off	(argcheck_strictness == 0)
-#define check_args_number (argcheck_strictness&01)
-#define check_args_type	(argcheck_strictness&02)
-#define check_args_all	(argcheck_strictness == 3)
-#define check_array_dims (array_arg_check&01) /* levels 1 and 3 */
-#define check_array_size (array_arg_check&02) /* levels 2 and 3 */
 
-#define check_var_set_used	(var_usage_check&01) /* levels 1 and 3 */
-#define check_var_unused	(var_usage_check&02) /* levels 2 and 3 */
-#define check_com_set_used	(com_usage_check&01) /* levels 1 and 3 */
-#define check_com_unused	(com_usage_check&02) /* levels 2 and 3 */
-#define check_ext_set_used	(ext_usage_check&01) /* levels 1 and 3 */
-#define check_ext_unused	(ext_usage_check&02) /* levels 2 and 3 */
+#define usage_var_any (usage_var_set_unused || usage_var_uninitialized \
+       || usage_var_unused)
+#define usage_com_any (usage_com_block_unused || usage_com_var_set_unused \
+       || usage_com_var_uninitialized || usage_com_var_unused)
 
-#define check_com_off	(comcheck_strictness == 0) /* no checking common */
-#define check_com_lengths (comcheck_strictness >= 2) /* match lengths */
-#define check_com_byname (comcheck_strictness == 3) /* match var by var */
-#define check_com_tree (volatile_flag&&check_com_set_used) /* Check undef errors */
-#define check_volatile_com (volatile_flag)/* Check not saved */
+#define COMCHECK_OFF (!(comcheck_by_name||comcheck_type||comcheck_length))
+
+#define check_com_tree (comcheck_volatile&&usage_com_block_volatile) /* Check undef errors */
 
-		/* Define macros for source format options */
-#define dec_tabs	(source_format&0x1)	/* DEC-style tabbed source */
-#define vms_include	(source_format&0x2)	/* VMS-style INCLUDE */
-#define unix_backslash	(source_format&0x4)	/* Treat backslash as escape */
 
 
 		/* Declare variables for commandline StrSettings */
@@ -808,10 +1025,22 @@ OPT(char,*out_fname,(char *)NULL);	/* Output filename */
 #ifdef ALLOW_INCLUDE
 OPT(char,*include_path,(char *)NULL);	/* An include-file directory */
 #endif
+#ifndef STANDARD_INTRINSICS
+OPT(char,*intrinsic_option_list,(char *)NULL); /* intrinsic fcn options */
+#endif
+OPT(char,*argcheck_warn_list,(char *)NULL);/* Arg mismatches to warn about */
+OPT(char,*arraycheck_warn_list,(char *)NULL);/* Arg arrayness warnings */
+OPT(char,*calltree_opt_list,(char *)NULL);/* Subprogram call graph options */
+OPT(char,*comcheck_warn_list,(char *)NULL);/* Common block mismatch warnings */
 OPT(char,*f77_warn_list,(char *)NULL); /* Non F77 extensions to warn about */
+OPT(char,*f90_warn_list,(char *)NULL); /* Non F90 extensions to warn about */
+OPT(char,*f95_warn_list,(char *)NULL); /* Non F95 old syntax to warn about */
+OPT(char,*makedcl_warn_list,(char *)NULL); /* Make-declarations options */
 OPT(char,*port_warn_list,(char *)NULL); /* Nonportable things to warn about */
 OPT(char,*pretty_warn_list,(char *)NULL); /* Misleading things to warn about */
+OPT(char,*source_form_list,(char *)NULL); /* Source format options */
 OPT(char,*trunc_warn_list,(char *)NULL); /* Truncation pitfalls to warn about */
+OPT(char,*usage_warn_list,(char *)NULL); /* Variable usages to warn about */
 
 SHARED unsigned
     line_num,		/* line num of current char */
@@ -889,7 +1118,7 @@ PROTO(int yyparse,  ( void ));
 	/* in ftnchek.c */
 PROTO(void msg_tail, ( char *s ));
 PROTO(void nonportable, ( unsigned lineno, unsigned colno, char *s ));
-PROTO(void nonstandard, ( unsigned lineno, unsigned colno ));
+PROTO(void nonstandard, ( unsigned lineno, unsigned colno, int f90, int f95 ));
 PROTO(void oops_message, ( int severity, unsigned lineno, unsigned colno, char *s ));
 PROTO(void oops_tail, ( char *s ));
 PROTO(void print_a_line, ( FILE *fd, char *line, unsigned num ));
@@ -917,10 +1146,13 @@ PROTO(void init_globals, ( void ));
 PROTO(unsigned long init_intrins_hashtab, ( void ));
 PROTO(void init_symtab, ( void ));
 PROTO(void note_filename, ( char *s ));
-PROTO(void set_intrinsic_options, ( int option ));
-
+#ifndef STANDARD_INTRINSICS
+PROTO(void set_intrinsic_options, ( void ));
+#endif
 #ifdef DEBUG_SIZES
 PROTO(void print_sizeofs, ( void ));
 #endif
 
+/* Misc. useful defines */
+#define MAX(X,Y) ((X)>(Y)?(X):(Y))
 
