@@ -2,10 +2,30 @@
 
 		Routines associated with printing of global symbol table info
 
-    Copyright (C) 1993 by Robert K. Moniot.
-    This program is free software.  Permission is granted to
-    modify it and/or redistribute it, retaining this notice.
-    No guarantees accompany this software.
+Copyright (c) 1999 by Robert K. Moniot.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of ftnchek shall not be used
+in advertising or otherwise to promote the sale, use or other dealings in
+this Software without prior written authorization from the author.
+
 
 	Shared functions defined:
 
@@ -87,8 +107,12 @@ PROTO(PRIVATE void visit_child_vcg,( Gsymtab *gsymt, int level ));
 
 #define pluralize(n) ((n)==1? "":"s")	/* singular/plural suffix for n */
 
-#define CMP_ERR_LIMIT 3	/* stop printing errors after this many */
-
+				/* Stop printing errors after limit is reached,
+				   unless limit is 0.  Increment error count
+				   even if don't print.
+				 */
+#define CASCADE_LIMIT(ERROR_COUNT) (++(ERROR_COUNT) > error_cascade_limit \
+			&& error_cascade_limit > 0)
 
 PRIVATE int cmp_error_count;
 PRIVATE int
@@ -104,7 +128,7 @@ cmp_error_head(name,message)
 	   */
 {
 		/* stop after limit: probably a cascade */
-	if(++cmp_error_count > CMP_ERR_LIMIT) {
+	if( CASCADE_LIMIT(cmp_error_count) ) {
 	  (void)fprintf(list_fd,"\n etc...");
 	  return TRUE;
 	}
@@ -194,7 +218,7 @@ arg_array_cmp(name,args1,args2)
 
 	n = (n1 > n2) ? n2: n1;		/* n = min(n1,n2) */
 
-	if (check_args_number && n1 != n2){
+	if (argcheck_argnumber && n1 != n2){
 	  cmp_error_count = 0;
 	  (void) cmp_error_head(name,"varying number of arguments:");
 
@@ -209,7 +233,7 @@ arg_array_cmp(name,args1,args2)
 	  arg_error_locate(args2);
 	}
 
-	if(check_args_type)
+	if(argcheck_argtype)
 	{	/* Look for type mismatches */
 	    cmp_error_count = 0;
 	    for (i=0; i<n; i++) {
@@ -411,7 +435,7 @@ arg_array_cmp(name,args1,args2)
 
 
 		 /* Check arrayness of args only if defn exists */
-	if(check_args_type && args1->is_defn ) {
+	if(argcheck_arrayness && args1->is_defn ) {
 	    cmp_error_count = 0;
 	    for (i=0; i<n; i++) {
 			/* Skip if class or datatype mismatch.  This
@@ -426,8 +450,8 @@ arg_array_cmp(name,args1,args2)
 		    if( a2[i].array_var ) {
 			if( a2[i].array_element ) {
 					/*   A. Actual arg is array elt */
-					/*	Warn on check_array_dims. */
-			    if(check_array_dims) {
+					/*	Warn on arraycheck_dims. */
+			    if(arraycheck_dims) {
 
 			      if(cmp_error_head(
 				      name," argument arrayness mismatch"))
@@ -477,9 +501,9 @@ arg_array_cmp(name,args1,args2)
 			  }
 
 			/* size = 0 or 1 means variable-dim: OK to differ */
-			  if( (check_array_size &&
+			  if( (arraycheck_size &&
 				  (size1>1 && size2>1 && cmpsize1 != cmpsize2))
-			     || (check_array_dims &&
+			     || (arraycheck_dims &&
 				  (dims1 != dims2)) ) {
 
 
@@ -601,15 +625,71 @@ arg_array_cmp(name,args1,args2)
 	}/* if( args1->is_defn ) */
 
 
-		 /* Check usage of args only if defn exists */
-	if(check_var_set_used && args1->is_defn) {
-
+		 /* Check usage of args only if defn exists.  Arg array
+		    1 is dummy args, array 2 is actual args.  */
+	if( (usage_arg_modified || usage_arg_alias_modified ||
+	     usage_array_alias_modified || usage_var_uninitialized ||
+	     usage_arg_common_modified || usage_array_common_modified)
+					&& args1->is_defn ) {
 	    cmp_error_count = 0;
 	    for (i=0; i<n; i++) {
 	      if(storage_class_of(a1[i].type) == class_VAR &&
 		 storage_class_of(a2[i].type) == class_VAR ) {
-		int nonlvalue_out = (a1[i].assigned_flag && !a2[i].is_lvalue),
-		    nonset_in = (a1[i].used_before_set && !a2[i].set_flag);
+		int nonlvalue_out = (a1[i].assigned_flag && !a2[i].is_lvalue);
+		int nonset_in = (a1[i].used_before_set && !a2[i].set_flag);
+		int alias_modified = (a1[i].set_flag && (a2[i].same_as != i));
+		int arg_alias_modified = (alias_modified && !a2[i].array_var);
+		int array_alias_modified = (alias_modified && a2[i].array_var);
+		int common_modified_as_arg, common_modified_as_com, /*maybe*/
+		    common_assigned_as_arg, common_assigned_as_com; /*for sure*/
+		int arg_common_modified, /*nonarray arg*/
+		    array_common_modified; /*array arg*/
+		char *common_alias_name = NULL;
+				/* See if arg aliased to common variable, and
+				   if so, is either one modified. */
+		common_modified_as_arg = common_modified_as_com = FALSE;
+		common_assigned_as_arg = common_assigned_as_com = FALSE;
+		if( a2[i].common_block != 0 ) {
+			/* Find out if block is defined in called routine */
+		  ComListHeader *clist = a2[i].common_block->info.comlist;
+		  while( clist != NULL ) {
+		    if( clist->module == args1->module ) {
+		      break;	/* found it */
+		    }
+		    clist = clist->next;
+		  }
+		  if( clist != NULL ) {	/* block is defined in called module */
+		      if( comcheck_by_name ) { /* Exact common: find the var */
+				/* It is not yet an error unless the block
+				   is also long enough to include the variable
+				   and the variable is modified in either
+				   place.
+				*/
+  			if( a2[i].common_index <= clist->numargs ) {
+			/* Don't forget that index goes from 1 to numargs.*/
+			  int j = a2[i].common_index - 1;
+			  common_alias_name = clist->com_list_array[j].name;
+			  common_modified_as_arg = a1[i].set_flag;
+			  common_assigned_as_arg = a1[i].assigned_flag;
+			  common_modified_as_com = clist->com_list_array[j].set;
+			  common_assigned_as_com = clist->com_list_array[j].assigned;
+			}
+		      }
+		      else {	/* Inexact common: just see if block or
+				   variable is modified.  Don't set
+				   assigned_as_com to always say "may be".
+				*/
+			common_modified_as_arg = a1[i].set_flag;
+			common_assigned_as_arg = a1[i].assigned_flag;
+			common_modified_as_com = clist->any_set;
+			common_assigned_as_com = FALSE;
+		      }
+		  } /* clist != NULL */
+		} /* a2[i].common_block != 0 */
+		arg_common_modified =
+		  ((common_modified_as_arg || common_modified_as_com) && !a2[i].array_var);
+		array_common_modified =
+		  ((common_modified_as_arg || common_modified_as_com) && a2[i].array_var);
 
 #if DEBUG_PGSYMTAB
 if(debug_latest) {
@@ -624,14 +704,22 @@ a2[i].set_flag);
 }
 #endif
 
-		if(nonlvalue_out || nonset_in) {
+		if( (usage_arg_modified && nonlvalue_out) ||
+		    (usage_var_uninitialized && nonset_in)||
+		    (usage_arg_alias_modified && arg_alias_modified)||
+		    (usage_array_alias_modified && array_alias_modified)||
+		    (usage_arg_common_modified && arg_common_modified)||
+		    (usage_array_common_modified && array_common_modified) ) {
 
 		  if(cmp_error_head(name," argument usage mismatch"))
 		     break;
 
 		  (void)fprintf(list_fd,"\n  at position %d:", i+1);
 
-		  if(nonlvalue_out) {
+			/* Usage case 1: Modifying arg that is constant or
+			   expression.
+			*/
+		  if(usage_arg_modified && nonlvalue_out) {
 #ifdef KEEP_ARG_NAMES
 		    (void)fprintf(list_fd,"\n    Dummy arg %s is modified",
 				  a1[i].name);
@@ -648,7 +736,10 @@ a2[i].set_flag);
 #endif
 		    arg_error_locate(args2);
 		  }
-		  else if(nonset_in) {
+
+			/* Usage case 2: Using arg that is not set.
+			*/
+		  if(usage_var_uninitialized && nonset_in) {
 
 #ifdef KEEP_ARG_NAMES
 		    (void)fprintf(list_fd,"\n    Dummy arg %s used before set",
@@ -665,11 +756,88 @@ a2[i].set_flag);
 		    (void)fprintf(list_fd,"\n    Actual arg not set");
 #endif
 		    arg_error_locate(args2);
+		  }
+
+			/* Usage case 3: Modifying arg that is the same as
+			   another arg.
+			*/
+		  if((usage_arg_alias_modified && arg_alias_modified)||
+		    (usage_array_alias_modified && array_alias_modified)) {
+#ifdef KEEP_ARG_NAMES
+		    (void)fprintf(list_fd,"\n    Dummy arg %s",
+				  a1[i].name);
+#else
+		    (void)fprintf(list_fd,"\n    Dummy arg");
+#endif
+		    if(! a1[i].assigned_flag)
+		      (void)fprintf(list_fd," may be");
+		    (void)fprintf(list_fd," modified");
+
+		    arg_error_locate(args1);
+
+#ifdef KEEP_ARG_NAMES
+		    (void)fprintf(list_fd,"\n    Actual arg %s",
+				  a2[i].name);
+		    if( a2[i].array_var )
+		      (void)fprintf(list_fd," may be");
+		    (void)fprintf(list_fd," same as %s at position %d",
+			   a2[a2[i].same_as].name,a2[i].same_as+1);
+#else
+		    (void)fprintf(list_fd,"\n    Actual arg");
+		    if( a2[i].array_var )
+		      (void)fprintf(list_fd," may be");
+		    (void)fprintf(list_fd," same as arg %d",
+				  a2[i].same_as+1);
+#endif
+		    arg_error_locate(args2);
+		  }
+
+			/* Usage case 4: Modifying arg that is the same as
+			   a variable in common.
+			*/
+		  if((usage_arg_common_modified && arg_common_modified)||
+		     (usage_array_common_modified && array_common_modified)) {
+#ifdef KEEP_ARG_NAMES
+		    (void)fprintf(list_fd,"\n    Dummy arg %s",
+				  a1[i].name);
+#else
+		    (void)fprintf(list_fd,"\n    Dummy arg");
+#endif
+		    if( common_modified_as_arg ) {
+		      if(! common_assigned_as_arg)
+			(void)fprintf(list_fd," may be");
+		      (void)fprintf(list_fd," modified");
+		    }
+		    arg_error_locate(args1);
+		    (void)fprintf(list_fd,
+		      "\n      is aliased to common block %s",
+		      a2[i].common_block->name);
+		    if( comcheck_by_name ) {
+		      (void)fprintf(list_fd," var #%ld %s",
+				    a2[i].common_index,common_alias_name);
+		    }
+		    else {
+		      (void)fprintf(list_fd," (somewhere)");
+		    }
+		    if( common_modified_as_com ) {
+		      (void)fprintf(list_fd," which %s modified",
+			    common_assigned_as_com? "is": "may be");
+		    }
+#ifdef KEEP_ARG_NAMES
+		    (void)fprintf(list_fd,"\n    Actual arg %s",
+				  a2[i].name);
+#else
+		    (void)fprintf(list_fd,"\n    Actual arg");
+#endif
+		    arg_error_locate(args2);
+		    (void)fprintf(list_fd,
+				"\n      is in common block %s",
+				  a2[i].common_block->name);
 		  }
 		}
 	      }
 	    }
-	}/*end if(check_var_set_used && args->is_defn) */
+	}/*end if( (usage_arg...) && args->is_defn) */
 
 }/* arg_array_cmp */
 
@@ -709,7 +877,8 @@ check_arglists(VOID)	/* Scans global symbol table for subprograms */
 		list_item = alist;
 		while(list_item != NULL){
 		    if(list_item->is_defn){
-			if(check_ext_set_used && num_defns > 0) {/* multiple defn */
+					/* report multiple defns */
+			if(usage_ext_multiply_defined && num_defns > 0) {
 			    if(num_defns == 1) {
 			      (void)fprintf(list_fd,
 				      "\nSubprogram %s multiply defined:\n    ",
@@ -740,9 +909,10 @@ check_arglists(VOID)	/* Scans global symbol table for subprograms */
 				   from unvisited library modules, skip. */
 		  if(irrelevant(defn_list))
 		    continue;
-				/* If no definitions found, report error
-				   unless -noext is given */
-		   if(check_ext_set_used) {
+
+				/* If no definitions found, report error */
+		   if( (usage_ext_undefined && glob_symtab[i].used_flag)
+		    || (usage_ext_declared_only && !glob_symtab[i].used_flag) ) {
 		     (void)fprintf(list_fd,
 			     "\nSubprogram %s never defined",
 			     glob_symtab[i].name);
@@ -770,7 +940,7 @@ check_arglists(VOID)	/* Scans global symbol table for subprograms */
 		    if(!glob_symtab[i].visited
 		       && datatype_of(glob_symtab[i].type) != type_BLOCK_DATA
 		       && !glob_symtab[i].library_module
-		       && check_ext_unused ) {
+		       && usage_ext_unused ) {
 			(void)fprintf(list_fd,"\nSubprogram %s never invoked",
 				glob_symtab[i].name);
 			(void)fprintf(list_fd, "\n    defined");
@@ -782,7 +952,7 @@ check_arglists(VOID)	/* Scans global symbol table for subprograms */
 			   no defn, 1st invocation will serve. Here treat
 			   use as actual arg like call.  Ignore calls & defns
 			   in unvisited library modules. */
-		if( check_args_type &&
+		if( argcheck_functype &&
 		   (defn_list->is_defn || !defn_list->external_decl)) {
 		  while(alist != NULL){
 			int typerrs = 0;
@@ -873,7 +1043,7 @@ check_comlists(VOID)        /* Scans global symbol table for common blocks */
 	  check_nameclash();
 	}
 
-	if(check_com_off)
+	if(COMCHECK_OFF)
 		return;
 
 	for (i=0; i<glob_symtab_top; i++){
@@ -942,7 +1112,7 @@ check_comlists(VOID)        /* Scans global symbol table for common blocks */
 		while( clist != NULL ){
 		    if(clist != model && !irrelevant(clist)) {
 
-			if(check_com_byname)
+			if(comcheck_by_name)
 			  com_cmp_strict(glob_symtab[i].name,model,clist);
 			else
 			  com_cmp_lax(glob_symtab[i].name,model,clist);
@@ -954,12 +1124,13 @@ check_comlists(VOID)        /* Scans global symbol table for common blocks */
 } /* check_comlists */
 
 
-
+		/* Common-list check for comcheck_type or comcheck_length
+		   (formerly strictness levels 1 & 2) */
 PRIVATE void
 #if HAVE_STDC
-com_cmp_lax(char *name, ComListHeader *c1, ComListHeader *c2)		/* Common-list check at levels 1 & 2 */
+com_cmp_lax(char *name, ComListHeader *c1, ComListHeader *c2)
 #else /* K&R style */
-com_cmp_lax(name,c1,c2)		/* Common-list check at levels 1 & 2 */
+com_cmp_lax(name,c1,c2)
      char *name;
      ComListHeader *c1,*c2;
 #endif /* HAVE_STDC */
@@ -1004,7 +1175,7 @@ com_cmp_lax(name,c1,c2)		/* Common-list check at levels 1 & 2 */
       words1 /= BpW;
       words2 /= BpW;
     }
-    if(check_com_lengths && words1 != words2) {
+    if(comcheck_length && words1 != words2) {
       (void)fprintf(list_fd,
 	      "\nCommon block %s: varying length:", name);
       (void)fprintf(list_fd,
@@ -1022,6 +1193,7 @@ com_cmp_lax(name,c1,c2)		/* Common-list check at levels 1 & 2 */
     }
 
 				/* Now check type matches */
+  if(comcheck_type) {
     done1=done2=FALSE;
     i1=i2=0;
     len1=len2=0;
@@ -1103,7 +1275,7 @@ com_cmp_lax(name,c1,c2)		/* Common-list check at levels 1 & 2 */
 	}
 
 	if(type_clash) {
-	     if(++numerrs > 3) {
+	     if( CASCADE_LIMIT(numerrs) ) {
 	       (void)fprintf(list_fd,"\netc...");
 	       break;		/* stop checking after third mismatch */
 	     }
@@ -1141,13 +1313,15 @@ com_cmp_lax(name,c1,c2)		/* Common-list check at levels 1 & 2 */
 	word1 += jump;
 	word2 += jump;
     }/* end for(;;) */
+  }/* end if(comcheck_type) */
 }
 
+	/* Common-list check name-by-name (formerly strictness level 3) */
 PRIVATE void
 #if HAVE_STDC
-com_cmp_strict(char *name, ComListHeader *c1, ComListHeader *c2)	/* Common-list check at level 3 */
+com_cmp_strict(char *name, ComListHeader *c1, ComListHeader *c2)
 #else /* K&R style */
-com_cmp_strict(name,c1,c2)	/* Common-list check at level 3 */
+com_cmp_strict(name,c1,c2)
 	char *name;
 	ComListHeader *c1, *c2;
 #endif /* HAVE_STDC */
@@ -1161,6 +1335,7 @@ com_cmp_strict(name,c1,c2)	/* Common-list check at level 3 */
 	ComListElement *a1 = c1->com_list_array,
 		       *a2 = c2->com_list_array;
 
+      if(comcheck_length) {
 	n = (n1 > n2) ? n2: n1;
 	if(n1 != n2){
 	  (void)fprintf(list_fd,
@@ -1175,6 +1350,7 @@ com_cmp_strict(name,c1,c2)	/* Common-list check at level 3 */
 		  n2,pluralize(n2));
 	  com_error_locate(c2);
         }
+      }
 #if DEBUG_PGSYMTAB
 if(debug_latest){
 (void)fprintf(list_fd,"block %s",name);
@@ -1190,6 +1366,7 @@ if(debug_latest){
 	            c2->filename);
 }
 #endif
+      if(comcheck_type) {
 	typerr = 0;
 	for (i=0; i<n; i++) {
 	  int t1 = datatype_of(a1[i].type),
@@ -1209,7 +1386,7 @@ if(debug_latest){
 
 	    if( t1 != t2 || s1 != s2 ) {
 				/* stop after limit: probably a cascade */
-			if(++typerr > CMP_ERR_LIMIT) {
+			if( CASCADE_LIMIT(typerr) ) {
 				(void)fprintf(list_fd,"\n etc...");
 				break;
 			}
@@ -1238,8 +1415,9 @@ if(debug_latest){
 
 		}/*end if(type or size mismatch)*/
 	}/*end for(i=0; i<n; i++)*/
+      }/* end if(comcheck_type) */
 
-
+      if(comcheck_dims) {
 	dimerr = 0;
 	for (i=0; i<n; i++){
 		unsigned long d1, d2, s1, s2;
@@ -1248,7 +1426,7 @@ if(debug_latest){
 			(d2=array_dims(a2[i].dimen_info))){
 
 				/* stop after limit: probably a cascade */
-			if(++dimerr > CMP_ERR_LIMIT) {
+			if( CASCADE_LIMIT(dimerr) ) {
 				(void)fprintf(list_fd,"\n etc...");
 				break;
 			}
@@ -1275,7 +1453,7 @@ if(debug_latest){
 			(s2=array_size(a2[i].dimen_info))){
 
 				/* stop after limit: probably a cascade */
-			if(++dimerr > CMP_ERR_LIMIT) {
+			if( CASCADE_LIMIT(dimerr) ) {
 				(void)fprintf(list_fd,"\n etc...");
 				break;
 			}
@@ -1300,7 +1478,7 @@ if(debug_latest){
 
 		}/*end if(array size mismatch)*/
 	}/*end for(i=0; i<n; i++)*/
-
+      }/*end if(comcheck_dims)*/
 }/*com_cmp_strict*/
 
 
@@ -1313,16 +1491,19 @@ void
 check_com_usage(VOID)
 {
 #ifdef DYNAMIC_TABLES		/* tables will be mallocked at runtime */
-    Gsymtab  **gsymlist;
+    Gsymtab  **gsymlist,**blocklist;
 #else
-    Gsymtab  *gsymlist[GLOBSYMTABSZ];
+    Gsymtab  *gsymlist[GLOBSYMTABSZ],*blocklist[GLOBSYMTABSZ];
 #endif
     int  i,numentries,numblocks;
     ComListHeader  *cmlist;
 
 #ifdef DYNAMIC_TABLES
       if( (gsymlist=(Gsymtab **)calloc(glob_symtab_top,sizeof(Gsymtab *)))
-	 == (Gsymtab **)NULL) {
+	 == (Gsymtab **)NULL
+	  ||(blocklist=(Gsymtab **)calloc(glob_symtab_top,sizeof(Gsymtab *)))
+	 == (Gsymtab **)NULL
+	  ) {
 	  oops_message(OOPS_FATAL,NO_LINE_NUM,NO_COL_NUM,
 		       "Cannot malloc space for common block list");
       }
@@ -1332,12 +1513,21 @@ check_com_usage(VOID)
     if(print_xref_list) {
 	for(i=numblocks=0;i<glob_symtab_top;i++){ /* loop thru global table */
 	   if (storage_class_of(glob_symtab[i].type) == class_COMMON_BLOCK){
+	     blocklist[numblocks++] = &glob_symtab[i];
+	   }
+	}
+	if(numblocks > 0) {
 
-	     cmlist = glob_symtab[i].info.comlist;
+	  sort_gsymbols(blocklist,numblocks); /* Sort the common block list */
+
+	  (void)fprintf(list_fd,
+		       "\n        Common block cross-reference list:\n");
+	  for(i=0; i<numblocks; i++) {
+	     cmlist = blocklist[i]->info.comlist;
 	     numentries=0;
 
 #ifdef DEBUG_COM_USAGE
-	     (void)fprintf(list_fd, "\n Common Block %s:\n",glob_symtab[i].name );
+	     (void)fprintf(list_fd, "\n Common Block %s:\n",blocklist[i]->name );
 #endif
 
 	     while (cmlist != NULL){ /* loop thru declarations */
@@ -1354,31 +1544,27 @@ check_com_usage(VOID)
 
 	     if (numentries >0){ /* print modules that declare this block*/
 
-	       if(numblocks++ == 0)
-		 (void)fprintf(list_fd,
-		       "\n        Common block cross-reference list:\n");
-
 	       (void)fprintf(list_fd, "\nCommon Block %s used in:\n" ,
-			glob_symtab[i].name );
+			blocklist[i]->name );
 
+				/* Sort modules that declare this block */
 	       sort_gsymbols(gsymlist,numentries);
 
 	       print_modules((unsigned)numentries,gsymlist);
 
-	     }  /* end of if */
+	     }  /* end of if numentries >0 */
 
 
-	   } /* end of if */
+	  } /* end of for i = 0 to numblocks */
 
-	} /* end of for */
-
-	if(numblocks > 0)
 	  (void)fprintf(list_fd,"\n");
 
+	} /* end of if numblocks > 0*/
+
     }/* end if print_xref_list */
 
 				/* Print out usage info */
-    if(com_usage_check > 0) {
+    if(usage_com_any) {
 	for(i=0;i<glob_symtab_top;i++){ /* loop thru global table */
 	   if (storage_class_of(glob_symtab[i].type) == class_COMMON_BLOCK){
 
@@ -1567,7 +1753,7 @@ com_block_usage(name,cl1)
 	  return;
 
      if(! (block_any_used || block_any_set) ) {	/* Totally unused */
-       if(check_com_unused) {
+       if(usage_com_block_unused) {
 	 IDENTIFY_COMBLOCK;
 	 (void)fprintf(list_fd," unused");
        }
@@ -1575,7 +1761,7 @@ com_block_usage(name,cl1)
      else {
 				/* If block used somewhere but not everywhere,
 				   report it. */
-        if(block_unused_somewhere && check_com_unused) {
+        if(block_unused_somewhere && usage_com_block_unused) {
 	  IDENTIFY_COMBLOCK;
 	  (void)fprintf(list_fd," unused");
 	  cur_cl = cl1;
@@ -1591,17 +1777,17 @@ com_block_usage(name,cl1)
 	  }
 	}/* end if block_unused_somewhere */
 
-	if(! check_com_byname) {
+	if(! comcheck_by_name) {
 				/* If not variablewise checking, just
 				   give general warnings. */
 	  if (!block_any_set){
-	    if(check_com_set_used) {
+	    if(usage_com_var_uninitialized) {
 	      IDENTIFY_COMBLOCK;
 	      (void)fprintf (list_fd," No elements are set, but some are used.");
 	    }
 	  }
 	  if (!block_any_used){
-	    if(check_com_set_used) {
+	    if(usage_com_var_set_unused) {
 	      IDENTIFY_COMBLOCK;
 	      (void)fprintf (list_fd," No elements are used, but some are set.");
 	    }
@@ -1609,7 +1795,30 @@ com_block_usage(name,cl1)
         }
 	else {	/* strictness == 3 */
 				/* Now go thru the details for each element */
+
+				/* First, malloc up a temporary list and
+				   copy ref_cl and its list there so the
+				   original is not clobbered (used later in
+				   arg usage checks for common aliasing)
+				*/
+	  ComListHeader *new_ref_cl;
+	  ComListElement *new_ref_list;
+	  if( (new_ref_cl=(ComListHeader *)calloc(1,sizeof(ComListHeader)))
+	      == (ComListHeader *)NULL ||
+	      (new_ref_list=(ComListElement *)calloc(ref_cl->numargs,sizeof(ComListElement)))
+	      == (ComListElement *)NULL ) {
+	    oops_message(OOPS_FATAL,NO_LINE_NUM,NO_COL_NUM,
+			 "Cannot alloc space for common block ref list");
+	  }
+	  *new_ref_cl = *ref_cl; /* Copy the header over to temporary */
+
 	  ref_list = ref_cl->com_list_array;
+	  for(j=0; j<ref_cl->numargs; j++) { /* Copy the array as well */
+	    new_ref_list[j] = ref_list[j];
+	  }
+	  ref_cl = new_ref_cl;	/* Now make the temporary the one we use */
+	  ref_list = new_ref_list;
+
 	  ref_cl->any_used = block_any_used;
 	  ref_cl->any_set = block_any_set;
 
@@ -1638,6 +1847,9 @@ com_block_usage(name,cl1)
 	    cur_cl = cur_cl->next;
 	  }
 	  com_element_usage(name, ref_cl, ref_list, ref_n);
+
+				/* Free up the temporary ref list */
+	  free(new_ref_cl); free(new_ref_list);
 	}
      }
 }
@@ -1660,7 +1872,7 @@ com_element_usage(name,  r_cl, r_list, r_num)
 
  	if (r_cl->any_used || r_cl->any_set){  /* if false block not used */
 
-	    if(check_com_set_used) {
+	    if(usage_com_var_uninitialized) {
 	      warnings = 0;
 	      for (i=0; i<r_num; i++){ /* Count used-not-set cases */
 		if (r_list[i].used && !r_list[i].set){
@@ -1689,7 +1901,7 @@ com_element_usage(name,  r_cl, r_list, r_num)
 	      }
 	    }
 
-	    if(check_com_unused) {
+	    if(usage_com_var_set_unused) {
 	      warnings = 0;
 	      for (i=0; i<r_num; i++){ /* Count set-not-used cases */
 		if (r_list[i].set && !r_list[i].used){
@@ -1718,11 +1930,11 @@ com_element_usage(name,  r_cl, r_list, r_num)
 	      }
 	    }
 
-	    warnings = 0;
-	    for (i=0,col=30; i<r_num; i++){
-	      if(!r_list[i].set && !r_list[i].used &&
-		 !r_list[i].used_before_set){
-		    if(check_com_unused) {
+	    if(usage_com_var_unused) {
+	      warnings = 0;
+	      for (i=0,col=30; i<r_num; i++){
+		if(!r_list[i].set && !r_list[i].used &&
+		   !r_list[i].used_before_set){
 		      IDENTIFY_COMBLOCK;
 		      if (warnings++ == 0 ){
 			(void)fprintf (list_fd,
@@ -1734,12 +1946,12 @@ com_element_usage(name,  r_cl, r_list, r_num)
 		      }
 		      (void)fprintf (list_fd, " %s",
 				     r_list[i].name);
-		    }
 		}
+	      }
 	    }
 	}
 	else{	/* This cannot be reached if called only when block is used */
-	  if(check_com_unused) {
+	  if(usage_com_block_unused) {
 	    IDENTIFY_COMBLOCK;
 	    (void)fprintf (list_fd," not used.");
 	  }
@@ -1754,9 +1966,6 @@ com_element_usage(name,  r_cl, r_list, r_num)
 PRIVATE int com_tree_error;
 PRIVATE int numvisited;
 
-#define PRUNE_CALLTREE	(!(call_tree_options & CALLTREE_NOPRUNE))
-#define SORT_CALLTREE   (!(call_tree_options & CALLTREE_NOSORT))
-
 void
 visit_children(VOID)
 {
@@ -1818,7 +2027,7 @@ visit_children(VOID)
        ) {
       (void)fprintf(list_fd,"\n  (no main program found)");
     }
-    else if(check_ext_set_used) {
+    else if(usage_ext_undefined) {
       (void)fprintf(list_fd,
 	"\nNo main program found");
     }
@@ -1865,12 +2074,12 @@ visit_children(VOID)
 
 			/* Check that common blocks retain definition
 			   status between uses. */
-  if(check_com_tree || check_volatile_com){
+  if(check_com_tree || comcheck_volatile){
     if(num_mains != 1) {
       if(check_com_tree)
 	(void)fprintf(list_fd,
 		"\nCommon definition check requires single main program");
-      if(check_volatile_com)
+      if(comcheck_volatile)
 	(void)fprintf(list_fd,
 		"\nCommon volatility check requires single main program");
     }
@@ -1883,7 +2092,7 @@ visit_children(VOID)
       for(i=0; i<glob_symtab_top; i++) {
 	if(storage_class_of(glob_symtab[i].type) == class_COMMON_BLOCK) {
 	  if( block_is_volatile(glob_symtab[i].info.comlist,main_module) ) {
-	    if(check_volatile_com) {
+	    if(comcheck_volatile) {
 	      (void)fprintf(list_fd,
 		   "\nCommon block %s is volatile",
 		   glob_symtab[i].name);
@@ -2082,7 +2291,7 @@ visit_child(gsymt,level)
 				/* If already visited, do not visit its
 				   children, but give note to reader if it
 				   has some. */
-  if(PRUNE_CALLTREE && gsymt->visited) {
+  if(call_tree_prune && gsymt->visited) {
     if(print_call_tree && child_list != NULL)
       (void)fprintf(list_fd," (see above)");
   }
@@ -2372,8 +2581,11 @@ print_crossrefs(VOID)
 	args = args->next;
       }
 
-      if(numcalls == (unsigned) 0)
+      if(numcalls == (unsigned) 0) {
 	(void)fprintf(list_fd," not called");
+	if(datatype_of(gsymlist[i]->type) == type_PROGRAM)
+	  (void)fprintf(list_fd," (main program)");
+      }
       else {
 	(void)fprintf(list_fd," called by:\n");
 	sort_gsymbols(modulelist,(int)numcalls); /* Sort the callers */
@@ -2542,7 +2754,7 @@ sort_child_list(child_list)
      ChildList *child_list;
 #endif /* HAVE_STDC */
 {
- if( SORT_CALLTREE ) {
+ if( call_tree_sort ) {
   ChildList *front,*prev,*next,*cl=child_list;
   Gsymtab *temp;
   prev = NULL;
