@@ -2,10 +2,30 @@
 
 		Routines associated with printing of local symbol table info
 
-    Copyright (C) 1993 by Robert K. Moniot.
-    This program is free software.  Permission is granted to
-    modify it and/or redistribute it, retaining this notice.
-    No guarantees accompany this software.
+Copyright (c) 1999 by Robert K. Moniot.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of ftnchek shall not be used
+in advertising or otherwise to promote the sale, use or other dealings in
+this Software without prior written authorization from the author.
+
 
 	Shared functions defined:
 
@@ -81,7 +101,9 @@ PROTO(PRIVATE void print_selected_declarations,( Lsymtab *sym_list[], int n,
 PROTO(PRIVATE int print_type_name,( int the_type, char *type_name, int
 			    the_size, Lsymtab *symt ));
 PROTO(PRIVATE int select_arguments,( Lsymtab *sym_entry ));
+#if 0 /* not currently used */
 PROTO(PRIVATE int select_commons,( Lsymtab *sym_entry ));
+#endif
 PROTO(PRIVATE int select_externals_by_name,( Lsymtab *sym_entry ));
 PROTO(PRIVATE int select_externals_by_type,( Lsymtab *sym_entry ));
 PROTO(PRIVATE int select_intrinsics_by_name,( Lsymtab *sym_entry ));
@@ -93,6 +115,12 @@ PROTO(PRIVATE int select_parameters,( Lsymtab *sym_entry ));
 PROTO(PRIVATE int select_statement_functions,( Lsymtab *sym_entry ));
 PROTO(PRIVATE int sf3_internal_name,( Lsymtab *sym_entry ));
 
+				/* Stop printing errors after limit is reached,
+				   unless limit is 0.  Increment error count
+				   even if don't print.
+				 */
+#define CASCADE_LIMIT(ERROR_COUNT) (++(ERROR_COUNT) > error_cascade_limit \
+			&& error_cascade_limit > 0)
 
 
 PRIVATE void
@@ -339,7 +367,7 @@ print_loc_symbols(curmodhash)
 
 
 				/* Print list of statement functions */
-    if(do_symtab || check_ext_unused) {
+    if(do_symtab || usage_ext_unused) {
 	   int i,n;
 
 	   for(i=0,n=0;i<loc_symtab_top;i++) {
@@ -353,7 +381,7 @@ print_loc_symbols(curmodhash)
 		(void)fprintf(list_fd,"\nStatement functions defined:\n");
 		imps += print_symbols(list_fd,sym_list,n,TRUE);
 	      }
-	      if(check_ext_unused) {
+	      if(usage_ext_unused) {
 		check_flags(sym_list,n,0,1,0,
 		 "Statement functions declared but never referenced",mod_name);
 	      }
@@ -404,7 +432,7 @@ print_loc_symbols(curmodhash)
     }/* End printing the namelists */
 				/* Process the variables */
 
-    if(do_symtab || var_usage_check) {
+    if(do_symtab || usage_var_any || pure_functions) {
 	int i,n;
 
 	for(i=0,n=0;i<loc_symtab_top;i++) {
@@ -412,7 +440,7 @@ print_loc_symbols(curmodhash)
 	       && (!loc_symtab[i].entry_point || this_is_a_function)) {
 		  sym_list[n++] = &loc_symtab[i];
 		  if(loc_symtab[i].argument && loc_symtab[i].set_flag) {
-		    if(++args_modified <= 3)
+		    if( ! CASCADE_LIMIT(args_modified) )
 			if(this_is_a_function && pure_functions) {
 			    identify_module(mod_name);
 			    (void)fprintf(list_fd,
@@ -424,7 +452,7 @@ print_loc_symbols(curmodhash)
 			}
 		  }
 		  if(loc_symtab[i].common_var && loc_symtab[i].set_flag) {
-		    if(++com_vars_modified <= 3)
+		    if( ! CASCADE_LIMIT(com_vars_modified) )
 			if(this_is_a_function && pure_functions) {
 			    identify_module(mod_name);
 			    (void)fprintf(list_fd,
@@ -437,7 +465,8 @@ print_loc_symbols(curmodhash)
 		  }
 	       }
 	}
-	if(args_modified > 3 || com_vars_modified > 3)
+	if(error_cascade_limit  > 0 &&
+	   (args_modified > error_cascade_limit || com_vars_modified > error_cascade_limit))
 	  if(this_is_a_function && pure_functions)
 	    (void)fprintf(list_fd,"\netc...");
 	if(n != 0) {
@@ -459,28 +488,30 @@ print_loc_symbols(curmodhash)
 	     ++warning_count;
 	}
 
-	if(var_usage_check) {
+	if(usage_var_unused || usage_var_set_unused || usage_var_uninitialized) {
 	  if(do_symtab || do_list)
 	    (void)fprintf(list_fd,"\n");
-	  if(check_var_unused) {
+	  if(usage_var_unused) {
 	    check_flags(sym_list,n,0,0,0,
 		      "Variables declared but never referenced",mod_name);
+	  }
+	  if(usage_var_set_unused) {
 	    check_flags(sym_list,n,0,1,0,
 		      "Variables set but never used",mod_name);
 	  }
-	  if(check_var_set_used) {
+	  if(usage_var_uninitialized) {
 	    check_flags(sym_list,n,1,0,1,
 		      "Variables used before set",mod_name);
 	    check_flags(sym_list,n,1,1,1,
 		      "Variables may be used before set",mod_name);
 	  }
 
-	}/*end if(var_usage_check)*/
+	}/*end if(usage_var_...)*/
 
 	if(do_symtab || do_list)
 	  (void)fprintf(list_fd,"\n");
 
-    }/* end if(do_symtab || var_usage_check) */
+    }/* end if(do_symtab || usage_var_any || pure_functions) */
 
 			/* List all undeclared vars & functions */
     if(decls_required || implicit_none) {
@@ -504,23 +535,27 @@ print_loc_symbols(curmodhash)
 	}
     }/*if(decls_required || implicit_none)*/
 
-			/* Under -f77, list any nonstandard intrinsics used */
-    if(f77_intrinsics) {
+			/* Under -f77/f90, list any nonstandard intrinsics used */
+    if(f77_intrinsics || f90_intrinsics) {
       int i,n;
       for(i=0,n=0;i<loc_symtab_top;i++) {
 	if(storage_class_of(loc_symtab[i].type) == class_SUBPROGRAM
 	   && loc_symtab[i].intrinsic &&
-	   (loc_symtab[i].info.intrins_info->intrins_flags & I_NONF77)) {
+	   (loc_symtab[i].info.intrins_info->intrins_flags & (f77_intrinsics?I_NONF77:I_NONF90))) {
 	  sym_list[n++] = &loc_symtab[i];
 	}
       }
       if(n != 0) {
 	sort_symbols(sym_list,n);
 	identify_module(mod_name);
-	(void)fprintf(list_fd,"Nonstandard intrinsic functions referenced:\n");
+	if(f77_intrinsics)
+	  (void)fprintf(list_fd,"Nonstandard");
+	else
+	  (void)fprintf(list_fd,"Non Fortran 90");
+	(void)fprintf(list_fd," intrinsic functions referenced:\n");
 	(void) print_symbols(list_fd,sym_list,n,FALSE);
       }
-    }/*if(f77_standard)*/
+    }/*if(f77_intrinsics || f90_intrinsics)*/
 
 
 		/* issue -f77 warning for identifiers
@@ -546,7 +581,7 @@ print_loc_symbols(curmodhash)
 
 	/* If -f77 flag given, list names with underscore or dollarsign */
 
-    if(f77_underscores || f77_dollarsigns) {
+    if(f77_underscores || f77_dollarsigns || f90_dollarsigns) {
 	int i,n;
 	for(i=0,n=0;i<loc_symtab_top;i++) {
 			/* Find all names with nonstd chars, but
@@ -566,7 +601,7 @@ print_loc_symbols(curmodhash)
 		   "Names containing nonstandard characters");
 	   (void) print_symbols(list_fd,sym_list,n,FALSE);
 	}
-    }/*if(f77_underscores || f77_dollarsigns)*/
+    }/*if(f77_underscores || f77_dollarsigns || f90_dollarsigns)*/
 
 			/* Print out clashes in first six chars of name */
     if(sixclash) {
@@ -586,7 +621,7 @@ print_loc_symbols(curmodhash)
 		   groups for mixed type. */
     if(port_mixed_equiv || port_mixed_size || local_wordsize==0) {
 	int i,j,n;
-	int imps=0;
+	int port_imps=0;
 	Lsymtab *equiv;
 
 		/* scan thru table for equivalenced variables */
@@ -680,12 +715,12 @@ print_loc_symbols(curmodhash)
 				      mixed_size?"sizes":
 				       "default and explicit size items");
 
-			imps += print_symbols(list_fd,sym_list,n,TRUE);
+			port_imps += print_symbols(list_fd,sym_list,n,TRUE);
 		    }
 		}
 	    }
 	}
-	if(imps != 0) {
+	if(port_imps != 0) {
 	     identify_module(mod_name);
 	     (void)fprintf(list_fd,"* Variable not declared.");
 	     (void)fprintf(list_fd," Type has been implicitly defined.\n");
@@ -699,16 +734,15 @@ print_loc_symbols(curmodhash)
 
 PRIVATE int
 #if HAVE_STDC
-has_nonalnum(char *s)	/* Returns TRUE if s contains a non-alphanumeric character
-		   and -f77, or if it has $ or _ and that is not allowed  */
+has_nonalnum(char *s)	/* Returns TRUE if s contains a $ or _ character
+			   and -f77 or -f90 is given. */
 #else /* K&R style */
-has_nonalnum(s)	/* Returns TRUE if s contains a non-alphanumeric character
-		   and -f77, or if it has $ or _ and that is not allowed  */
+has_nonalnum(s)
    char *s;
 #endif /* HAVE_STDC */
 {
    while( *s != '\0' ) {
-     if( (f77_dollarsigns && (*s) == '$')
+     if( ((f77_dollarsigns||f90_dollarsigns) && (*s) == '$')
       || (f77_underscores && (*s) == '_') )
        return TRUE;
      s++;
@@ -938,8 +972,8 @@ print_arg_array(arglist)	/* prints type and flag info for arguments */
 	if(arglist->external_decl || arglist->actual_arg)
 	  count = 0;
 	a = arglist->arg_array;
-	(void)fprintf(list_fd,"\nArg list in module %s file %s line %u:",
-		arglist->module->name, arglist->filename, arglist->line_num);
+	(void)fprintf(list_fd,"\nArg list in module %s (topfile %s) file %s line %u:",
+		arglist->module->name, arglist->topfile, arglist->filename, arglist->line_num);
 	(void)fprintf(list_fd,"\n\tdef%d call%d ext%d arg%d",
 		arglist->is_defn,
 		arglist->is_call,
@@ -1202,7 +1236,18 @@ debug_symtabs(VOID) 	/* Debugging output: hashtable and symbol tables */
 {
 #ifdef DEBUG_SYMTABS
   if(debug_loc_symtab) {
-    (void)fprintf(list_fd,"\n Debugging of local symbol table disabled");
+    int i;
+    (void)fprintf(list_fd,"\nLocal Symbol table:\n");
+    (void)fprintf(list_fd,"\nCommon variables:\n");
+    for(i=0; i < loc_symtab_top; i++) {
+      if(loc_symtab[i].common_block != NULL) {
+	(void)fprintf(list_fd,"\n%10s: block %s[%d]",
+		      loc_symtab[i].name,
+		      loc_symtab[i].common_block->name,
+		      loc_symtab[i].common_index);
+      }
+    }
+    (void)fprintf(list_fd,"\n");
     return;
   }
 
@@ -1302,31 +1347,26 @@ PROTO(PRIVATE char * get_size_expression,( Lsymtab *symt ));
 #define ACTUAL_SIZE(p)		(get_size((p),sym_type))
 #endif
 
-#define DCL_FLAGS_DECLARATIONS				0x0001
-#define DCL_FLAGS_ONLY_UNDECLARED			0x0002
-#define DCL_FLAGS_COMPACT				0x0004
-#define DCL_FLAGS_USE_CONTINUATIONS			0x0008
-#define DCL_FLAGS_KEYWORDS_LOWERCASE			0x0010
-#define DCL_FLAGS_VARIABLES_AND_CONSTANTS_LOWERCASE	0x0020
-#define DCL_FLAGS_EXCLUDE_SFTRAN3_INTERNAL_VARIABLES	0x0040
-#define DCL_FLAGS_ASTERISK_COMMENT_CHARACTER		0x0080
-#define DCL_FLAGS_LOWERCASE_COMMENT_CHARACTER		0x0100
-#define DCL_NO_ARRAY_DIMENSIONS				0x0200
+#define ANY_DCL_DECLARATIONS() (dcl_declarations\
+  ||dcl_only_undeclared\
+  ||dcl_compact||dcl_use_continuations\
+  ||dcl_keywords_lowercase\
+  ||dcl_vars_and_consts_lowercase\
+  ||dcl_excl_sftran3_internal_vars\
+  ||dcl_asterisk_comment_character\
+  ||dcl_lowercase_comment_character\
+  ||dcl_no_array_dimensions)
+#define DECLARE_ONLY_UNDECLARED() (dcl_only_undeclared)
+#define DECLARE_COMPACT()	(dcl_compact)
+#define NO_CONTINUATION_LINES() (!(dcl_use_continuations))
+#define SF3_DECLARATIONS()	(dcl_excl_sftran3_internal_vars)
+#define ASTERISK_COMMENT_CHAR()	(dcl_asterisk_comment_character)
+#define KEYWORDS_LOWERCASE()	(dcl_keywords_lowercase)
+#define LOWERCASE_COMMENT_CHARACTER() (dcl_lowercase_comment_character)
+#define VARIABLES_AND_CONSTANTS_LOWERCASE() (dcl_vars_and_consts_lowercase)
+#define ARRAY_VARS_DIMENSIONED() (!(dcl_no_array_dimensions))
+
 #define COLUMN_WIDTH		13
-#define DECLARE_ONLY_UNDECLARED() (make_dcls & DCL_FLAGS_ONLY_UNDECLARED)
-#define DECLARE_COMPACT()	(make_dcls & DCL_FLAGS_COMPACT)
-#define NO_CONTINUATION_LINES() (!(make_dcls & DCL_FLAGS_USE_CONTINUATIONS))
-#define SF3_DECLARATIONS()	\
-		(make_dcls & DCL_FLAGS_EXCLUDE_SFTRAN3_INTERNAL_VARIABLES)
-#define ASTERISK_COMMENT_CHAR()	\
-		(make_dcls & DCL_FLAGS_ASTERISK_COMMENT_CHARACTER)
-#define KEYWORDS_LOWERCASE()	(make_dcls & DCL_FLAGS_KEYWORDS_LOWERCASE)
-#define LOWERCASE_COMMENT_CHARACTER() \
-		(make_dcls & DCL_FLAGS_LOWERCASE_COMMENT_CHARACTER)
-#define VARIABLES_AND_CONSTANTS_LOWERCASE() \
-		(make_dcls & DCL_FLAGS_VARIABLES_AND_CONSTANTS_LOWERCASE)
-#define ARRAY_VARS_DIMENSIONED() \
-		(!(make_dcls & DCL_NO_ARRAY_DIMENSIONS))
 
 #ifndef FIRST_VARIABLE_COLUMN
 #define FIRST_VARIABLE_COLUMN	26      /* to match Extended PFORT Verifier */
@@ -1518,11 +1558,9 @@ make_declarations(sym_list,mod_name)
     char begin[72+1+2+1];
     int len_current_filename = strlen(current_filename);
 
-    if (!make_dcls)
+    if ( ! ANY_DCL_DECLARATIONS() )
 	return;
 
-    make_dcls |= DCL_FLAGS_DECLARATIONS; /* any non-zero value selects */
-
     if (LOWERCASE_COMMENT_CHARACTER())
 	comment_char = 'c';
     else if (ASTERISK_COMMENT_CHAR())
@@ -2287,6 +2325,7 @@ select_arguments(sym_entry)
 }
 
 
+#if 0				/* this function not currently used */
 PRIVATE int
 #if HAVE_STDC
 select_commons(Lsymtab *sym_entry)
@@ -2301,6 +2340,7 @@ select_commons(sym_entry)
     else
 	return (0);
 }
+#endif /*0*/
 
 
 PRIVATE int
