@@ -1,5 +1,32 @@
 /* project.c:
-	Project-file I/O routines.  Routines included:
+	Project-file I/O routines.
+
+Copyright (c) 1999 by Robert K. Moniot.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of ftnchek shall not be used
+in advertising or otherwise to promote the sale, use or other dealings in
+this Software without prior written authorization from the author.
+
+
+  Routines included:
 
 	Shared routines:
 	   void proj_file_out() writes data from symbol table to project file.
@@ -168,11 +195,16 @@ proj_file_out(fd)
 	  WRITE_NUM(" class",(long)storage_class_of(sym_list[i]->type));
 	  WRITE_NUM(" type",(long)datatype_of(sym_list[i]->type));
 	  WRITE_NUM(" size",(long)sym_list[i]->size);
+		/* Flag values stored are cumulative only for current file
+		   so they will not depend on what files were previously
+		   read in current run.  When project file is read, flags
+		   will be ORed into Gsymtab as is done in process_lists.
+		*/
 	  (void)fprintf(fd," flags %d %d %d %d %d %d %d %d",
-		  sym_list[i]->used_flag,
-		  sym_list[i]->set_flag,
-		  sym_list[i]->invoked_as_func,
-		  sym_list[i]->declared_external,
+		  sym_list[i]->used_this_file,
+		  sym_list[i]->set_this_file,
+		  sym_list[i]->invoked_as_func_this_file,
+		  sym_list[i]->declared_external_this_file,
 		  /* N.B. library_module included here but is not restored */
 		  sym_list[i]->library_module,
 		  0,	/* Flags for possible future use */
@@ -294,6 +326,15 @@ proj_alist_out(gsymt,fd,do_defns,locally_defined)
 		     arg[i].info.array_dim: 0 );
 	WRITE_NUM(" dims",(long)array_dims(diminfo));
 	WRITE_NUM(" elts",(long)array_size(diminfo));
+	{ char *cblk;
+	  if( arg[i].common_block == (Gsymtab *)NULL )
+	    cblk = "-";	/* place holder if no block name */
+	  else
+	    cblk = arg[i].common_block->name;
+	  WRITE_STR(" cblk",cblk);
+	}
+	WRITE_NUM(" cndx",(long)arg[i].common_index);
+	WRITE_NUM(" same",(long)arg[i].same_as);
 	(void)fprintf(fd," flags %d %d %d %d %d %d %d %d",
 		arg[i].is_lvalue,
 		arg[i].set_flag,
@@ -537,6 +578,9 @@ proj_arg_info_in(fd,filename,is_defn)
     unsigned numargs,iarg,arg_num,arg_class,arg_type,arg_dims;
     unsigned long arg_elts;
     long arg_size;
+    char arg_common_block[MAXNAME+1];
+    long arg_common_index;
+    int arg_same_as;
     unsigned			/* Flags for arguments */
 		arg_is_lvalue,
 		arg_set_flag,
@@ -739,6 +783,9 @@ alist_class,alist_type,alist_line);
 	READ_LONG(" size",arg_size);
 	READ_NUM(" dims",arg_dims);
 	READ_LONG(" elts",arg_elts);
+	READ_STR(" cblk",arg_common_block);
+	READ_LONG(" cndx",arg_common_index);
+	READ_NUM(" same",arg_same_as);
 	if(fscanf(fd," flags %d %d %d %d %d %d %d %d",
 		&arg_is_lvalue,
 		&arg_set_flag,
@@ -765,6 +812,23 @@ alist_class,alist_type,alist_line);
 	alist[iarg].info.array_dim = array_dim_info(arg_dims,arg_elts);
 	alist[iarg].type = type_byte(arg_class,arg_type);
 	alist[iarg].size = arg_size;
+	if( strcmp(arg_common_block,"-") == 0 ) { /* indicator for "none" */
+	  alist[iarg].common_block = (Gsymtab *)NULL;
+	}
+	else {
+	  Gsymtab *gsymt;
+	  int h = hash_lookup(arg_common_block);
+			/* Block may not have been seen yet since common
+			   decls come at end of project file.  There is a
+			   risk that block could end up in Gsymtab with no
+			   definition, but only if project file is corrupt.
+			 */
+	  if( (gsymt = hashtab[h].com_glob_symtab) == NULL)
+	    gsymt = install_global(h,type_COMMON_BLOCK,class_COMMON_BLOCK);
+	  alist[iarg].common_block = gsymt;
+	}
+	alist[iarg].common_index = arg_common_index;
+	alist[iarg].same_as = (short)arg_same_as;
 	alist[iarg].is_lvalue = arg_is_lvalue;
 	alist[iarg].set_flag = arg_set_flag;
 	alist[iarg].assigned_flag = arg_assigned_flag;
