@@ -3,10 +3,29 @@
 	Shared declarations for symbol-table routines.  Note: uses
 	declarations in ftnchek.h.
 
-    Copyright (C) 1993 by Robert K. Moniot.
-    This program is free software.  Permission is granted to
-    modify it and/or redistribute it, retaining this notice.
-    No guarantees accompany this software.
+Copyright (c) 1999 by Robert K. Moniot.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of ftnchek shall not be used
+in advertising or otherwise to promote the sale, use or other dealings in
+this Software without prior written authorization from the author.
 
 
 */
@@ -32,6 +51,18 @@
 #define SEQ_EXEC     5
 #define SEQ_END      6
 
+				/* Separate sequence defs to allow checking
+				   f90 sequence requirements. */
+#define F90_SEQ_HEADER		1
+#define F90_SEQ_USE		2
+#define F90_SEQ_IMPLICIT_NONE	3
+#define F90_SEQ_IMPLICIT	4
+#define F90_SEQ_SPECIF		5
+#define F90_SEQ_EXEC		6
+#define F90_SEQ_CONTAINS	7
+#define F90_SEQ_INTERNAL	8
+#define F90_SEQ_END		9
+
 		/* Definitions of symbol table information */
 
 /*	Token subclasses (classes are in tokdefs.h)
@@ -251,19 +282,14 @@ char *implicit_len_text[('Z'-'A'+1)+2];
 	/* Declaration of Token data structure.  N.B. do not change without
 	   consulting preamble of fortran.y for uses with nonterminals.
 	 */
-			/* temporary equivs for future separate fields */
-#ifndef TOKEN_TYPE_SEPARATE /* define this to unequivalence type==class */
-#define TOK_type class
-#endif
-#ifndef TOKEN_FLAGS_SEPARATE /* define this to unequivalence flags==subclass */
-#define TOK_flags subclass
-#endif
-				/* these are for array-bounds tokens */
-#define TOK_dims TOK_type
-#define TOK_elts TOK_flags
 
-#define TOK_start TOK_type
-#define TOK_end  TOK_flags
+			/* temporary equivs for future separate fields */
+				/* these are for array-bounds tokens */
+#define TOK_dims class
+#define TOK_elts subclass
+
+#define TOK_start class
+#define TOK_end  subclass
 
 
 struct tokstruct {
@@ -271,21 +297,17 @@ struct tokstruct {
 		long integer;
 		DBLVAL dbl;
 		char *string;
-	} value;
+	} value;		/* Value of constant */
 	struct tokstruct
 	  *left_token,		/* Left child in expr tree */
 	  *next_token;		/* Right child or next in linked list */
 	char *src_text;		/* Original text string of token */
-	long size;
-#ifndef TOK_type
-	long TOK_type;
-#endif
-#ifndef TOK_flags
-	unsigned long TOK_flags;
-#endif
-	long class,subclass;
-	unsigned line_num;	/* Line and column where token occurred */
-	unsigned col_num : 8;
+	long class,subclass;	/* Token category and subcategory */
+	long size;		/* sizeof(datatype) */
+	long TOK_type;		/* Storage class & data type of identifier */
+	unsigned TOK_flags:32;	/* Exprtype flags (see defns below) */
+	unsigned line_num:32;	/* Line where token occurred */
+	unsigned col_num : 8;	/* Column where token occurred */
 	unsigned size_is_adjustable : 1;
 	unsigned size_is_expression : 1;
 };
@@ -359,8 +381,11 @@ typedef union {		/* InfoUnion: misc info about symtab entry */
 typedef struct {	/* ArgListElement: holds subprog argument data */
 	char *name;		/* name of dummy arg or text of actual arg */
 	InfoUnion info;
+	struct gSymtEntry *common_block; /* block it belongs to if any */
+	long common_index;	/* index in block */
 	long size;
 	BYTE type;
+	short same_as;	/* index if two actual arguments the same */
 	unsigned is_lvalue: 1,
 		 set_flag: 1,
 		 assigned_flag: 1,
@@ -427,26 +452,7 @@ typedef struct TLHead {	/* TokenListHeader: head node of token list */
 
 
 			/* Structure for intrinsic-function info */
-typedef struct IInfo{
-	char *name;
-	short num_args,
-	      arg_type,
-	      result_type;
-	unsigned short
-	      intrins_flags;	/* nonstandard,  mixed arg types */
-} IntrinsInfo;
 
-			/* Structure for parameter info */
-typedef struct PInfo{
-	char *src_text;		/* source text of parameter value */
-	union {
-	  long integer;		/* integer value */
-	  DBLVAL dbl;		/* float value */
-	  char *string;		/* character string value */
-	} value;
-	int seq_num;		/* position in parameter definitions */
-} ParamInfo;
-	
 	/* Define special num_args values for intrinsics that have
 	   variable numbers of arguments. */
 #define I_1or2	(-1)		/* 1 or 2 arguments */
@@ -481,6 +487,40 @@ typedef struct PInfo{
 #define I_EXTRA		0x2000	/* commonly found extra intrinsics */
 #define I_VMS		0x4000	/* VMS systems only */
 #define I_UNIX		0x8000	/* Unix systems only */
+#define I_NONF90	0x10000	/* Not in F90 standard */
+
+				/* Define flag type big enough for 17 bits */
+#if (SIZEOF_SHORT > 2)
+typedef unsigned short intrins_flags_t;
+#else
+#if (SIZEOF_INT > 2)
+typedef unsigned int intrins_flags_t;
+#else
+#if (SIZEOF_LONG > 2)
+typedef unsigned long intrins_flags_t;
+#endif
+#endif
+#endif
+typedef struct IInfo{
+	char *name;
+	short num_args,
+	      arg_type,
+	      result_type;
+	intrins_flags_t
+	      intrins_flags;	/* nonstandard,  mixed arg types */
+} IntrinsInfo;
+
+			/* Structure for parameter info */
+typedef struct PInfo{
+	char *src_text;		/* source text of parameter value */
+	union {
+	  long integer;		/* integer value */
+	  DBLVAL dbl;		/* float value */
+	  char *string;		/* character string value */
+	} value;
+	int seq_num;		/* position in parameter definitions */
+} ParamInfo;
+	
 
 			/* Structure for call-tree child list */
 typedef struct childlist {
@@ -500,6 +540,12 @@ typedef struct lSymtEntry{
 	  TokenListHeader *toklist; /* for namelist & common block makedecls */
 	} src;
 	struct lSymtEntry *equiv_link;	/* Link for equivalence lists */
+		/* common_block is a ptr to block if this is a common
+		   variable, and common_index is its position (starting
+		   from 1).  For block, common_index is the count of
+		   variables in it. */
+	struct gSymtEntry *common_block;
+	long common_index;
 	long size;		/* Size of object in bytes */
 	BYTE  type;		/* Type & storage class: see macros below */
 			/* Flags */
@@ -548,7 +594,14 @@ typedef struct gSymtEntry{	/* Global symbol table element */
 	     visited_somewhere: 1, /* some entry point of module is in call tree */
 	     defined: 1,	/* is defined somewhere */
 	     defined_in_include: 1,
-	     declared_external: 1;
+	     declared_external: 1,
+			/* The following flags are for project-file use.
+			   They get reset when a file is opened and accumulate
+			   values as file is read.  */
+	     used_this_file: 1,
+	     set_this_file: 1,
+	     invoked_as_func_this_file: 1,
+	     declared_external_this_file: 1;
 } Gsymtab;
 
 
@@ -627,7 +680,9 @@ typedef struct PSpace {
 
 
 		/* Defns used by expression type propagation mechanisms
-		   in fortran.y and exprtype.c  The flags go in token.subclass
+		   in fortran.y and exprtype.c  The flags go in token.TOK_flags
+		   Make sure size of TOK_flags declared above suffices for
+		   largest item in the list below.
 		 */
 
 #define make_true(flag,x) ((x) |= (flag))		/* x.flag <-- true   */
@@ -650,11 +705,12 @@ typedef struct PSpace {
 #define USED_BEFORE_SET		0x1000  /* id used beforre set */
 #define COMPLEX_FLAG		0x2000	/* remembers complex_const_allowed */
 #define CHAR_ID_EXPR		0x4000	/* char var or array elt not substr */
-#define IN_ASSIGN		0x8000	/* for tracking assgn stmt lhs */
-#define COMMA_FLAG 		0x10000/* keeps track of extra or missing
+#define DIM_BOUND_EXPR		0x8000	/* no array or func ref (5.1.1.1) */
+#define IN_ASSIGN 		0x10000	/* for tracking assgn stmt lhs */
+#define COMMA_FLAG		0x20000/* keeps track of extra or missing
 				   	  commas in exprlists (obsolete) */
-#define NONSTD_USAGE_FLAG	0x20000	/* concentrator for -f77 warnings */
-#define SYNTAX_ERROR_FLAG	0x40000	/* concentrator for syntax errors */
+#define NONSTD_USAGE_FLAG	0x40000	/* concentrator for -f77 warnings */
+#define SYNTAX_ERROR_FLAG	0x80000	/* concentrator for syntax errors */
 
 #ifdef DYNAMIC_TABLES		/* tables will be mallocked at runtime */
 SYM_SHARED
@@ -779,7 +835,7 @@ PROTO(void def_label,( Token *lab ));
 #endif
 PROTO(void def_namelist,( Token *id, Token *list ));
 PROTO(void def_namelist_item,( Token *id ));
-PROTO(void def_parameter,( Token *id, Token *val ));
+PROTO(void def_parameter,( Token *id, Token *val, int noparen ));
 PROTO(void def_stmt_function,( Token *id, Token *args ));
 PROTO(void do_ASSIGN,( Token *id ));
 PROTO(void do_assigned_GOTO,( Token *id ));
