@@ -14,7 +14,7 @@ FILE=$3
 	if [ ! -d ${OKAY} ] ;
 	then
 	    echo "Creating directory ${OKAY}" ;
-	    mkdir ${OKAY};
+	    /bin/mkdir ${OKAY};
 	fi
 	if [ ! -f ${OKAY}/${FILE} ] ;
 	then
@@ -23,7 +23,7 @@ FILE=$3
 	fi
 	if /usr/bin/cmp ${OKAY}/${FILE} ${FILE} ;
 	then
-	    /bin/rm -f ${FILE} ;
+	    /bin/rm -f -f ${FILE} ;
 	    echo '.' | gawk '{printf("%s",$1);}' ;
 	else
 	    touch CHECK_FAILED
@@ -32,7 +32,7 @@ FILE=$3
 	    if [ ! -d ${VARY} ] ;
 	    then
 		echo "Creating directory ${VARY}" ;
-		mkdir ${VARY} ;
+		/bin/mkdir ${VARY} ;
 	    fi
 	    /bin/mv ${FILE} ${VARY}/${FILE} ;
 	    /usr/bin/diff ${OKAY}/${FILE} ${VARY}/${FILE} ;
